# -*- coding: utf-8 -*-

""" #+begin_org
* ~[Summary]~ :: A =CS-Unit= as equivalent of facter in py and remotely with rpyc.
#+end_org """

####+BEGIN: b:py3:cs:file/dblockControls :classification "cs-u"
""" #+begin_org
* [[elisp:(org-cycle)][| /Control Parameters Of This File/ |]] :: dblk ctrls classifications=cs-u
#+BEGIN_SRC emacs-lisp
(setq-local b:dblockControls t) ; (setq-local b:dblockControls nil)
(put 'b:dblockControls 'py3:cs:Classification "cs-u") ; one of cs-mu, cs-u, cs-lib, bpf-lib, pyLibPure
#+END_SRC
#+RESULTS:
: cs-u
#+end_org """
####+END:

####+BEGIN: b:prog:file/proclamations :outLevel 1
""" #+begin_org
* *[[elisp:(org-cycle)][| Proclamations |]]* :: Libre-Halaal Software --- Part Of BISOS ---  Poly-COMEEGA Format.
** This is Libre-Halaal Software. © Neda Communications, Inc. Subject to AGPL.
** It is part of BISOS (ByStar Internet Services OS)
** Best read and edited  with Blee in Poly-COMEEGA (Polymode Colaborative Org-Mode Enhance Emacs Generalized Authorship)
#+end_org """
####+END:

####+BEGIN: b:prog:file/particulars :authors ("./inserts/authors-mb.org")
""" #+begin_org
* *[[elisp:(org-cycle)][| Particulars |]]* :: Authors, version
** This File: /bisos/git/bxRepos/bisos-pip/myLinkedIn/py3/bisos/myLinkedIn/linkedin_csu.py
** File True Name: /bisos/git/auth/bxRepos/bisos-pip/myLinkedIn/py3/bisos/myLinkedIn/linkedin_csu.py
** Authors: Mohsen BANAN, http://mohsen.banan.1.byname.net/contact
#+end_org """
####+END:

####+BEGIN: b:py3:file/particulars-csInfo :status "inUse"
""" #+begin_org
* *[[elisp:(org-cycle)][| Particulars-csInfo |]]*
#+end_org """
import typing
csInfo: typing.Dict[str, typing.Any] = { 'moduleName': ['linkedin_csu'], }
csInfo['version'] = '202505100819'
csInfo['status']  = 'inUse'
csInfo['panel'] = 'linkedin_csu-Panel.org'
csInfo['groupingType'] = 'IcmGroupingType-pkged'
csInfo['cmndParts'] = 'IcmCmndParts[common] IcmCmndParts[param]'
####+END:

""" #+begin_org
* [[elisp:(org-cycle)][| ~Description~ |]] :: [[file:/bisos/git/auth/bxRepos/blee-binders/bisos-core/COMEEGA/_nodeBase_/fullUsagePanel-en.org][BISOS COMEEGA Panel]]
This a =Cs-Unit= for running the equivalent of facter in py and remotely with rpyc.
With BISOS, it is used in CMDB remotely.

** Relevant Panels:
** Status: In use with BISOS
** /[[elisp:(org-cycle)][| Planned Improvements |]]/ :
*** TODO complete fileName in particulars.
#+end_org """

####+BEGIN: b:prog:file/orgTopControls :outLevel 1
""" #+begin_org
* [[elisp:(org-cycle)][| Controls |]] :: [[elisp:(delete-other-windows)][(1)]] | [[elisp:(show-all)][Show-All]]  [[elisp:(org-shifttab)][Overview]]  [[elisp:(progn (org-shifttab) (org-content))][Content]] | [[file:Panel.org][Panel]] | [[elisp:(blee:ppmm:org-mode-toggle)][Nat]] | [[elisp:(bx:org:run-me)][Run]] | [[elisp:(bx:org:run-me-eml)][RunEml]] | [[elisp:(progn (save-buffer) (kill-buffer))][S&Q]]  [[elisp:(save-buffer)][Save]]  [[elisp:(kill-buffer)][Quit]] [[elisp:(org-cycle)][| ]]
** /Version Control/ ::  [[elisp:(call-interactively (quote cvs-update))][cvs-update]]  [[elisp:(vc-update)][vc-update]] | [[elisp:(bx:org:agenda:this-file-otherWin)][Agenda-List]]  [[elisp:(bx:org:todo:this-file-otherWin)][ToDo-List]]

#+end_org """
####+END:

####+BEGIN: b:py3:file/workbench :outLevel 1
""" #+begin_org
* [[elisp:(org-cycle)][| Workbench |]] :: [[elisp:(python-check (format "/bisos/venv/py3/bisos3/bin/python -m pyclbr %s" (bx:buf-fname))))][pyclbr]] || [[elisp:(python-check (format "/bisos/venv/py3/bisos3/bin/python -m pydoc ./%s" (bx:buf-fname))))][pydoc]] || [[elisp:(python-check (format "/bisos/pipx/bin/pyflakes %s" (bx:buf-fname)))][pyflakes]] | [[elisp:(python-check (format "/bisos/pipx/bin/pychecker %s" (bx:buf-fname))))][pychecker (executes)]] | [[elisp:(python-check (format "/bisos/pipx/bin/pycodestyle %s" (bx:buf-fname))))][pycodestyle]] | [[elisp:(python-check (format "/bisos/pipx/bin/flake8 %s" (bx:buf-fname))))][flake8]] | [[elisp:(python-check (format "/bisos/pipx/bin/pylint %s" (bx:buf-fname))))][pylint]]  [[elisp:(org-cycle)][| ]]
#+end_org """
####+END:

####+BEGIN: b:py3:cs:orgItem/basic :type "=PyImports= "  :title "*Py Library IMPORTS*" :comment "-- Framework and External Packages Imports"
""" #+begin_org
*  _[[elisp:(blee:menu-sel:outline:popupMenu)][±]]_ _[[elisp:(blee:menu-sel:navigation:popupMenu)][Ξ]]_ [[elisp:(outline-show-branches+toggle)][|=]] [[elisp:(bx:orgm:indirectBufOther)][|>]] *[[elisp:(blee:ppmm:org-mode-toggle)][|N]]*  =PyImports=  [[elisp:(outline-show-subtree+toggle)][||]] *Py Library IMPORTS* -- Framework and External Packages Imports  [[elisp:(org-cycle)][| ]]
#+end_org """
####+END:

# import os
import collections
# import pathlib
# import invoke

####+BEGIN: b:py3:cs:framework/imports :basedOn "classification"
""" #+begin_org
*  _[[elisp:(blee:menu-sel:outline:popupMenu)][±]]_ _[[elisp:(blee:menu-sel:navigation:popupMenu)][Ξ]]_ [[elisp:(outline-show-branches+toggle)][|=]] [[elisp:(bx:orgm:indirectBufOther)][|>]] *[[elisp:(blee:ppmm:org-mode-toggle)][|N]]*  CsFrmWrk   [[elisp:(outline-show-subtree+toggle)][||]] *Imports* =Based on Classification=cs-u=
#+end_org """
from bisos import b
from bisos.b import cs
from bisos.b import b_io
from bisos.common import csParam

import collections
####+END:

# from bisos.facter import facter
# from bisos.banna import bannaPortNu

import pathlib
import re

# from telemetry import Telemetry

from bisos.myLinkedIn import linkedinUtils
from bisos.myLinkedIn import connections
from bisos.myLinkedIn import invitations
from bisos.myLinkedIn import messages

from bisos.myLinkedIn import messagesMaildir

import logging
logger = logging.getLogger(__name__)
#logger.setLevel(logging.INFO)



####+BEGIN: b:py3:cs:orgItem/basic :type "=Executes=  "  :title "CSU-Lib Executions" :comment "-- cs.invOutcomeReportControl"
""" #+begin_org
*  _[[elisp:(blee:menu-sel:outline:popupMenu)][±]]_ _[[elisp:(blee:menu-sel:navigation:popupMenu)][Ξ]]_ [[elisp:(outline-show-branches+toggle)][|=]] [[elisp:(bx:orgm:indirectBufOther)][|>]] *[[elisp:(blee:ppmm:org-mode-toggle)][|N]]*  =Executes=   [[elisp:(outline-show-subtree+toggle)][||]] CSU-Lib Executions -- cs.invOutcomeReportControl  [[elisp:(org-cycle)][| ]]
#+end_org """
####+END:

cs.invOutcomeReportControl(cmnd=True, ro=True)

####+BEGIN: b:py3:cs:orgItem/section :title "Common Parameters Specification" :comment "based on cs.param.CmndParamDict -- As expected from CSU-s"
""" #+begin_org
*  _[[elisp:(blee:menu-sel:outline:popupMenu)][±]]_ _[[elisp:(blee:menu-sel:navigation:popupMenu)][Ξ]]_ [[elisp:(outline-show-branches+toggle)][|=]] [[elisp:(bx:orgm:indirectBufOther)][|>]] *[[elisp:(blee:ppmm:org-mode-toggle)][|N]]*  /Section/    [[elisp:(outline-show-subtree+toggle)][||]] *Common Parameters Specification* based on cs.param.CmndParamDict -- As expected from CSU-s  [[elisp:(org-cycle)][| ]]
#+end_org """
####+END:

####+BEGIN: b:py3:cs:func/typing :funcName "commonParamsSpecify" :comment "~CSU Specification~" :funcType "ParSpc" :deco ""
""" #+begin_org
*  _[[elisp:(blee:menu-sel:outline:popupMenu)][±]]_ _[[elisp:(blee:menu-sel:navigation:popupMenu)][Ξ]]_ [[elisp:(outline-show-branches+toggle)][|=]] [[elisp:(bx:orgm:indirectBufOther)][|>]] *[[elisp:(blee:ppmm:org-mode-toggle)][|N]]*  F-T-ParSpc [[elisp:(outline-show-subtree+toggle)][||]] /commonParamsSpecify/  ~CSU Specification~  [[elisp:(org-cycle)][| ]]
#+end_org """
def commonParamsSpecify(
####+END:
        csParams: cs.param.CmndParamDict,
) -> None:
    csParams.parDictAdd(
        parName='myLinkedInBase',
        parDescription="Path to the directory where myLinkedIn files and directories are stored or will be written.",
        parDataType=None,
        parDefault="~/bpos/usageEnvs/selected/myLinkedIn/selected",
        parChoices=[],
        argparseShortOpt=None,
        argparseLongOpt='--myLinkedInBase',
    )
    csParams.parDictAdd(
        parName='dataExportDir',
        parDescription="Path to the directory where Basic_LinkedInDataExport.zip is unzipped in.",
        parDataType=None,
        parDefault="~/bpos/usageEnvs/selected/myLinkedIn/selected/LinkedInDataExport",
        parChoices=[],
        argparseShortOpt=None,
        argparseLongOpt='--dataExportDir',
    )
    csParams.parDictAdd(
        parName='vcardsDir',
        parDescription="Path to the directory where .vcf files are stored or will be written.",
        parDataType=None,
        parDefault="~/bpos/usageEnvs/selected/myLinkedIn/selected/VCards",
        parChoices=[],
        argparseShortOpt=None,
        argparseLongOpt='--vcardsDir',
    )
    csParams.parDictAdd(
        parName='maildir',
        parDescription="Path to the directory where maildir files are stored or will be written.",
        parDataType=None,
        parDefault="~/bpos/usageEnvs/selected/myLinkedIn/selected/maildir",
        parChoices=[],
        argparseShortOpt=None,
        argparseLongOpt='--maildir',
    )
    csParams.parDictAdd(
        parName='extent',
        parDescription="The extent to which a command should invoke others.",
        parDataType=None,
        parDefault="default",
        parChoices=["default", "full", "minimal"],
        argparseShortOpt=None,
        argparseLongOpt='--extent',
    )


####+BEGIN: blee:bxPanel:foldingSection :outLevel 0 :sep nil :title "Direct Command Services" :anchor ""  :extraInfo "Examples and CSs"
""" #+begin_org
*  _[[elisp:(blee:menu-sel:outline:popupMenu)][±]]_ _[[elisp:(blee:menu-sel:navigation:popupMenu)][Ξ]]_ [[elisp:(outline-show-branches+toggle)][|=]] [[elisp:(bx:orgm:indirectBufOther)][|>]] *[[elisp:(blee:ppmm:org-mode-toggle)][|N]]*     [[elisp:(outline-show-subtree+toggle)][| _Direct Command Services_: |]]  Examples and CSs  [[elisp:(org-shifttab)][<)]] E|
#+end_org """
####+END:

####+BEGIN: b:py3:cs:cmnd/classHead :cmndName "examples_csu" :comment "" :parsMand "" :parsOpt "" :argsMin 0 :argsMax 0 :pyInv ""
""" #+begin_org
*  _[[elisp:(blee:menu-sel:outline:popupMenu)][±]]_ _[[elisp:(blee:menu-sel:navigation:popupMenu)][Ξ]]_ [[elisp:(outline-show-branches+toggle)][|=]] [[elisp:(bx:orgm:indirectBufOther)][|>]] *[[elisp:(blee:ppmm:org-mode-toggle)][|N]]*  CmndSvc-   [[elisp:(outline-show-subtree+toggle)][||]] <<examples_csu>>  =verify= ro=cli   [[elisp:(org-cycle)][| ]]
#+end_org """
class examples_csu(cs.Cmnd):
    cmndParamsMandatory = [ ]
    cmndParamsOptional = [ ]
    cmndArgsLen = {'Min': 0, 'Max': 0,}

    @cs.track(fnLoc=True, fnEntry=True, fnExit=True)
    def cmnd(self,
             rtInv: cs.RtInvoker,
             cmndOutcome: b.op.Outcome,
    ) -> b.op.Outcome:

        failed = b_io.eh.badOutcome
        callParamsDict = {}
        if self.invocationValidate(rtInv, cmndOutcome, callParamsDict, None).isProblematic():
            return failed(cmndOutcome)
####+END:
        self.cmndDocStr(f""" #+begin_org
** [[elisp:(org-cycle)][| *CmndDesc:* | ]]  Basic example command.
        #+end_org """)

        od = collections.OrderedDict
        cmnd = cs.examples.cmndEnter
        literal = cs.examples.execInsert

        cs.examples.menuChapter('=Direct Interface Commands=')

        # fromFilePars = od([('fromFile', fileName), ('cache', 'True')])

        oneExportZipFile="~/bpos/usageEnvs/selected/myLinkedIn/selected/Basic_LinkedInDataExport.zip"
        oneLinkedInBase="~/bpos/usageEnvs/selected/myLinkedIn/selected"
        oneDataExportDir="~/bpos/usageEnvs/selected/myLinkedIn/selected/LinkedInDataExport"
        oneVCardsDir="~/bpos/usageEnvs/selected/myLinkedIn/selected/VCards"
        oneMaildir="~/bpos/usageEnvs/selected/myLinkedIn/selected/maildir"

        downloadedExportZipFile="~/Downloads/Basic_LinkedInDataExport.zip"
        downloadedLinkedinBase="~/Downloads"

        cs.examples.menuSection('/Show Selected as target of symlink/')
        literal(f"readlink -f {oneVCardsDir}")
        literal(f"ls -l {oneVCardsDir}")
        literal(f"ls -l {oneExportZipFile}")

        cs.examples.menuSection('/fullUpdate:: Using Exported Basic_LinkedInDataExport.zip File (Creates VCards and maildir)/')

        cmnd('fullUpdate',
             args=oneExportZipFile)

        cmnd('fullUpdate',
             pars=od([('myLinkedInBase', downloadedLinkedinBase),]),
             args=downloadedExportZipFile)
        
        cs.examples.menuSection('/exportedPrep:: Prepare the LinkedInDataExport Base -- Unzips arg0 in dataExportDir/')

        cmnd('exportedPrep',
             pars=od([('dataExportDir', oneDataExportDir),]),
             args=oneExportZipFile)

        cs.examples.menuSection('/vcardsGenerate:: -- Generate Initial VCards/')

        cmnd('vcardsGenerate',
             pars=od([('vcardsDir', oneVCardsDir),]),
             args=f"{oneLinkedInBase}/Connections.csv")

        cs.examples.menuSection('/vcardsInvitations:: -- Augment VCards/')

        cmnd('vcardsInvitations',
             pars=od([('vcardsDir', oneVCardsDir),]),
             args=f"{oneLinkedInBase}/Invitations.csv")

        cs.examples.menuSection('/maildirMessage:: vcardsMessages:: -- Create maildir from messages.csv/')

        cmnd('vcardsMessages',
             pars=od([('vcardsDir', oneVCardsDir),]),
             args=f"{oneLinkedInBase}/messages.csv")

        cmnd('maildirMessages',
             pars=od([('maildir', oneMaildir),]),
             args=f"{oneLinkedInBase}/messages.csv")

        return(cmndOutcome)


def extract_date_from_filename(zip_path: pathlib.Path) -> str:
    """Extracts the date portion (MM-DD-YYYY) from a LinkedIn ZIP filename."""
    match = re.search(r'LinkedInDataExport_(\d{2}-\d{2}-\d{4})', zip_path.name)
    if match:
        return match.group(1)
    raise ValueError(f"Date not found in filename: {zip_path.name}")

def refresh_symlink_dir(symlink_path: pathlib.Path, target_dir: pathlib.Path) -> None:
    """
    Replace or create a symbolic link pointing to target_dir.

    Args:
        symlink_path (Path): The path where the symlink will be created.
        target_dir (Path): The directory the symlink should point to.
    """
    symlink_path = symlink_path.expanduser()
    target_dir = target_dir.expanduser().resolve(strict=True)

    if symlink_path.exists() or symlink_path.is_symlink():
        symlink_path.unlink()

    symlink_path.symlink_to(target_dir, target_is_directory=True)
    logger.info(f"Symlink refreshed: {symlink_path} → {target_dir}")

####+BEGIN: b:py3:cs:cmnd/classHead :cmndName "fullUpdate" :comment "" :extent "verify" :ro "cli" :parsMand "" :parsOpt "extent myLinkedInBase" :argsMin 1 :argsMax 1 :pyInv ""
""" #+begin_org
*  _[[elisp:(blee:menu-sel:outline:popupMenu)][±]]_ _[[elisp:(blee:menu-sel:navigation:popupMenu)][Ξ]]_ [[elisp:(outline-show-branches+toggle)][|=]] [[elisp:(bx:orgm:indirectBufOther)][|>]] *[[elisp:(blee:ppmm:org-mode-toggle)][|N]]*  CmndSvc-   [[elisp:(outline-show-subtree+toggle)][||]] <<fullUpdate>>  =verify= parsOpt=extent myLinkedInBase argsMin=1 argsMax=1 ro=cli   [[elisp:(org-cycle)][| ]]
#+end_org """
class fullUpdate(cs.Cmnd):
    cmndParamsMandatory = [ ]
    cmndParamsOptional = [ 'extent', 'myLinkedInBase', ]
    cmndArgsLen = {'Min': 1, 'Max': 1,}

    @cs.track(fnLoc=True, fnEntry=True, fnExit=True)
    def cmnd(self,
             rtInv: cs.RtInvoker,
             cmndOutcome: b.op.Outcome,
             extent: typing.Optional[str]=None,  # Cs Optional Param
             myLinkedInBase: typing.Optional[str]=None,  # Cs Optional Param
             argsList: typing.Optional[list[str]]=None,  # CsArgs
    ) -> b.op.Outcome:

        failed = b_io.eh.badOutcome
        callParamsDict = {'extent': extent, 'myLinkedInBase': myLinkedInBase, }
        if self.invocationValidate(rtInv, cmndOutcome, callParamsDict, argsList).isProblematic():
            return failed(cmndOutcome)
        cmndArgsSpecDict = self.cmndArgsSpec()
        extent = csParam.mappedValue('extent', extent)
        myLinkedInBase = csParam.mappedValue('myLinkedInBase', myLinkedInBase)
####+END:
        self.cmndDocStr(f""" #+begin_org
** [[elisp:(org-cycle)][| *CmndDesc:* | ]]
        #+end_org """)

        fileToExportedZip = self.cmndArgsGet("0", cmndArgsSpecDict, argsList)
        path_exportedZip = pathlib.Path(fileToExportedZip).expanduser().resolve(strict=True)

        assert  path_exportedZip.exists(), f"Missing {path_exportedZip}"

        path_myLinkedInBase = pathlib.Path(myLinkedInBase).expanduser().resolve(strict=True)

        g_parDict = cs.globalContext.get().icmParamDictGet().parDictGet()

        # if myLinkedInBase was not specified on the command line
        # and the default does not exist, use the parent of ExportedZip file.
        try:
            g_parDict['myLinkedInBase']
        except  KeyError:
            # An optional param was not specified.
            if not path_myLinkedInBase.exists():
                path_myLinkedInBase = path_exportedZip.parent

        # assert  path_myLinkedInBase.exists(), f"Missing {path_myLinkedInBase}"
        if not path_myLinkedInBase.exists(): return failed(cmndOutcome, f"Missing {path_myLinkedInBase}")

        myLinkedInBase = str(path_myLinkedInBase)

        logger.info(f"cmnd={fullUpdate} -- optPar:: myLinkedInBase={myLinkedInBase}  arg0:: exportedZip={path_exportedZip}")

        if exportedPrep().pyCmnd(
            dataExportDir=f"{myLinkedInBase}/LinkedInDataExport",
            argsList=argsList,
        ).isProblematic(): return(b_io.eh.badOutcome(cmndOutcome))

        if vcardsGenerate().pyCmnd(
            myLinkedInBase=myLinkedInBase,
            vcardsDir=f"{myLinkedInBase}/VCards",
            argsList=[f"{myLinkedInBase}/LinkedInDataExport/Connections.csv"],
        ).isProblematic(): return(b_io.eh.badOutcome(cmndOutcome))

        if vcardsInvitations().pyCmnd(
            myLinkedInBase=myLinkedInBase,
            vcardsDir=f"{myLinkedInBase}/VCards",
            argsList=[f"{myLinkedInBase}/LinkedInDataExport/Invitations.csv"],
        ).isProblematic(): return(b_io.eh.badOutcome(cmndOutcome))

        if extent == "full":
            if vcardsMessages().pyCmnd(
                myLinkedInBase=myLinkedInBase,
                vcardsDir=f"{myLinkedInBase}/VCards",
                argsList=[f"{myLinkedInBase}/LinkedInDataExport/messages.csv"],
            ).isProblematic(): return(b_io.eh.badOutcome(cmndOutcome))

        if maildirMessages().pyCmnd(
            myLinkedInBase=myLinkedInBase,
            maildir=f"{myLinkedInBase}/maildir",
            argsList=[f"{myLinkedInBase}/LinkedInDataExport/messages.csv"],
        ).isProblematic(): return(b_io.eh.badOutcome(cmndOutcome))

        return cmndOutcome.set(opResults=None,)

####+BEGIN: b:py3:cs:method/args :methodName "cmndArgsSpec" :methodType "anyOrNone" :retType "bool" :deco "default" :argsList "self"
    """ #+begin_org
**  _[[elisp:(blee:menu-sel:outline:popupMenu)][±]]_ _[[elisp:(blee:menu-sel:navigation:popupMenu)][Ξ]]_ [[elisp:(outline-show-branches+toggle)][|=]] [[elisp:(bx:orgm:indirectBufOther)][|>]] *[[elisp:(blee:ppmm:org-mode-toggle)][|N]]*  Mtd-T-anyOrNone [[elisp:(outline-show-subtree+toggle)][||]] /cmndArgsSpec/ deco=default  deco=default  [[elisp:(org-cycle)][| ]]
    #+end_org """
    @cs.track(fnLoc=True, fnEntry=True, fnExit=True)
    def cmndArgsSpec(self, ):
####+END:
        """
***** Cmnd Args Specification
"""
        cmndArgsSpecDict = cs.CmndArgsSpecDict()

        cmndArgsSpecDict.argsDictAdd(
            argPosition="0",
            argName="pathToExportedZip",
            argDefault='',
            argChoices=[],
            argDescription="Path to the LinkedIn Basic_LinkedInDataExport_DATE.zip file."
        )

        return cmndArgsSpecDict

####+BEGIN: b:py3:cs:cmnd/classHead :cmndName "exportedPrep" :comment "" :extent "verify" :ro "cli" :parsMand "" :parsOpt "dataExportDir" :argsMin 1 :argsMax 1 :pyInv ""
""" #+begin_org
*  _[[elisp:(blee:menu-sel:outline:popupMenu)][±]]_ _[[elisp:(blee:menu-sel:navigation:popupMenu)][Ξ]]_ [[elisp:(outline-show-branches+toggle)][|=]] [[elisp:(bx:orgm:indirectBufOther)][|>]] *[[elisp:(blee:ppmm:org-mode-toggle)][|N]]*  CmndSvc-   [[elisp:(outline-show-subtree+toggle)][||]] <<exportedPrep>>  =verify= parsOpt=dataExportDir argsMin=1 argsMax=1 ro=cli   [[elisp:(org-cycle)][| ]]
#+end_org """
class exportedPrep(cs.Cmnd):
    cmndParamsMandatory = [ ]
    cmndParamsOptional = [ 'dataExportDir', ]
    cmndArgsLen = {'Min': 1, 'Max': 1,}

    @cs.track(fnLoc=True, fnEntry=True, fnExit=True)
    def cmnd(self,
             rtInv: cs.RtInvoker,
             cmndOutcome: b.op.Outcome,
             dataExportDir: typing.Optional[str]=None,  # Cs Optional Param
             argsList: typing.Optional[list[str]]=None,  # CsArgs
    ) -> b.op.Outcome:

        failed = b_io.eh.badOutcome
        callParamsDict = {'dataExportDir': dataExportDir, }
        if self.invocationValidate(rtInv, cmndOutcome, callParamsDict, argsList).isProblematic():
            return failed(cmndOutcome)
        cmndArgsSpecDict = self.cmndArgsSpec()
        dataExportDir = csParam.mappedValue('dataExportDir', dataExportDir)
####+END:
        self.cmndDocStr(f""" #+begin_org
** [[elisp:(org-cycle)][| *CmndDesc:* | ]]  Unzips arg0 in dataExportDir, if myLinkedInBase does not exist.
        #+end_org """)

        fileToExportedZip = self.cmndArgsGet("0", cmndArgsSpecDict, argsList)
        path_exportedZip = pathlib.Path(fileToExportedZip).expanduser().resolve()
        if not path_exportedZip.exists(): return failed(cmndOutcome, f"Missing {path_exportedZip}")

        path_dataExportDir = pathlib.Path(dataExportDir).expanduser().resolve()

        logger.info(f"cmnd={exportedPrep} -- optPar:: dataExportDir={path_dataExportDir}  arg0:: exportedZip={path_exportedZip}")

        if path_dataExportDir.is_dir():
            logger.info(f"dataExportDir={path_dataExportDir} exists, creation skipped")
        else:
            linkedinUtils.Common.unzip_file(path_exportedZip, path_dataExportDir)

        return cmndOutcome.set(opResults=f"dataExportDir={path_dataExportDir}",)


####+BEGIN: b:py3:cs:method/args :methodName "cmndArgsSpec" :methodType "anyOrNone" :retType "bool" :deco "default" :argsList "self"
    """ #+begin_org
**  _[[elisp:(blee:menu-sel:outline:popupMenu)][±]]_ _[[elisp:(blee:menu-sel:navigation:popupMenu)][Ξ]]_ [[elisp:(outline-show-branches+toggle)][|=]] [[elisp:(bx:orgm:indirectBufOther)][|>]] *[[elisp:(blee:ppmm:org-mode-toggle)][|N]]*  Mtd-T-anyOrNone [[elisp:(outline-show-subtree+toggle)][||]] /cmndArgsSpec/ deco=default  deco=default  [[elisp:(org-cycle)][| ]]
    #+end_org """
    @cs.track(fnLoc=True, fnEntry=True, fnExit=True)
    def cmndArgsSpec(self, ):
####+END:
        """
***** Cmnd Args Specification
"""
        cmndArgsSpecDict = cs.CmndArgsSpecDict()

        cmndArgsSpecDict.argsDictAdd(
            argPosition="0",
            argName="pathToExportedZip",
            argDefault='',
            argChoices=[],
            argDescription="Path to the LinkedIn Basic_LinkedInDataExport_DATE.zip file."
        )

        return cmndArgsSpecDict


####+BEGIN: b:py3:cs:cmnd/classHead :cmndName "vcardsGenerate" :comment "" :extent "verify" :ro "cli" :parsMand "" :parsOpt "myLinkedInBase vcardsDir" :argsMin 1 :argsMax 1 :pyInv ""
""" #+begin_org
*  _[[elisp:(blee:menu-sel:outline:popupMenu)][±]]_ _[[elisp:(blee:menu-sel:navigation:popupMenu)][Ξ]]_ [[elisp:(outline-show-branches+toggle)][|=]] [[elisp:(bx:orgm:indirectBufOther)][|>]] *[[elisp:(blee:ppmm:org-mode-toggle)][|N]]*  CmndSvc-   [[elisp:(outline-show-subtree+toggle)][||]] <<vcardsGenerate>>  =verify= parsOpt=myLinkedInBase vcardsDir argsMin=1 argsMax=1 ro=cli   [[elisp:(org-cycle)][| ]]
#+end_org """
class vcardsGenerate(cs.Cmnd):
    cmndParamsMandatory = [ ]
    cmndParamsOptional = [ 'myLinkedInBase', 'vcardsDir', ]
    cmndArgsLen = {'Min': 1, 'Max': 1,}

    @cs.track(fnLoc=True, fnEntry=True, fnExit=True)
    def cmnd(self,
             rtInv: cs.RtInvoker,
             cmndOutcome: b.op.Outcome,
             myLinkedInBase: typing.Optional[str]=None,  # Cs Optional Param
             vcardsDir: typing.Optional[str]=None,  # Cs Optional Param
             argsList: typing.Optional[list[str]]=None,  # CsArgs
    ) -> b.op.Outcome:

        failed = b_io.eh.badOutcome
        callParamsDict = {'myLinkedInBase': myLinkedInBase, 'vcardsDir': vcardsDir, }
        if self.invocationValidate(rtInv, cmndOutcome, callParamsDict, argsList).isProblematic():
            return failed(cmndOutcome)
        cmndArgsSpecDict = self.cmndArgsSpec()
        myLinkedInBase = csParam.mappedValue('myLinkedInBase', myLinkedInBase)
        vcardsDir = csParam.mappedValue('vcardsDir', vcardsDir)
####+END:
        self.cmndDocStr(f""" #+begin_org
** [[elisp:(org-cycle)][| *CmndDesc:* | ]]  Takes in Connections.csv, creates vcards, returns count
        #+end_org """)


        inFile = self.cmndArgsGet("0", cmndArgsSpecDict, argsList)
        path_inFile = pathlib.Path(inFile).expanduser().resolve()
        if not path_inFile.exists(): return failed(cmndOutcome, f"Missing {path_inFile}")

        if vcardsDir is not None:
            path_vcardsDir = pathlib.Path(vcardsDir).expanduser().resolve(strict=False)
            path_vcardsDir.mkdir(exist_ok=True)

        logger.info(f"cmnd={vcardsGenerate} -- optPar:: vcardsDir={vcardsDir}  arg0:: Connections.csv={path_inFile}")

        generation = connections.LinkedInConnections(path_inFile)
        generation.load()
        count = generation.create_vcards(path_vcardsDir)

        return cmndOutcome.set(opResults=f"vcardsDir={path_vcardsDir} count={count}",)


####+BEGIN: b:py3:cs:method/args :methodName "cmndArgsSpec" :methodType "anyOrNone" :retType "bool" :deco "default" :argsList "self"
    """ #+begin_org
**  _[[elisp:(blee:menu-sel:outline:popupMenu)][±]]_ _[[elisp:(blee:menu-sel:navigation:popupMenu)][Ξ]]_ [[elisp:(outline-show-branches+toggle)][|=]] [[elisp:(bx:orgm:indirectBufOther)][|>]] *[[elisp:(blee:ppmm:org-mode-toggle)][|N]]*  Mtd-T-anyOrNone [[elisp:(outline-show-subtree+toggle)][||]] /cmndArgsSpec/ deco=default  deco=default  [[elisp:(org-cycle)][| ]]
    #+end_org """
    @cs.track(fnLoc=True, fnEntry=True, fnExit=True)
    def cmndArgsSpec(self, ):
####+END:
        """
***** Cmnd Args Specification
"""
        cmndArgsSpecDict = cs.CmndArgsSpecDict()

        cmndArgsSpecDict.argsDictAdd(
            argPosition="0",
            argName="inFile",
            argDefault='',
            argChoices=[],
            argDescription="Path to the Exported LinkedIn connections file."
        )

        return cmndArgsSpecDict



####+BEGIN: b:py3:cs:cmnd/classHead :cmndName "vcardsInvitations" :comment "" :extent "verify" :ro "cli" :parsMand "" :parsOpt "myLinkedInBase vcardsDir" :argsMin 1 :argsMax 1 :pyInv ""
""" #+begin_org
*  _[[elisp:(blee:menu-sel:outline:popupMenu)][±]]_ _[[elisp:(blee:menu-sel:navigation:popupMenu)][Ξ]]_ [[elisp:(outline-show-branches+toggle)][|=]] [[elisp:(bx:orgm:indirectBufOther)][|>]] *[[elisp:(blee:ppmm:org-mode-toggle)][|N]]*  CmndSvc-   [[elisp:(outline-show-subtree+toggle)][||]] <<vcardsInvitations>>  =verify= parsOpt=myLinkedInBase vcardsDir argsMin=1 argsMax=1 ro=cli   [[elisp:(org-cycle)][| ]]
#+end_org """
class vcardsInvitations(cs.Cmnd):
    cmndParamsMandatory = [ ]
    cmndParamsOptional = [ 'myLinkedInBase', 'vcardsDir', ]
    cmndArgsLen = {'Min': 1, 'Max': 1,}

    @cs.track(fnLoc=True, fnEntry=True, fnExit=True)
    def cmnd(self,
             rtInv: cs.RtInvoker,
             cmndOutcome: b.op.Outcome,
             myLinkedInBase: typing.Optional[str]=None,  # Cs Optional Param
             vcardsDir: typing.Optional[str]=None,  # Cs Optional Param
             argsList: typing.Optional[list[str]]=None,  # CsArgs
    ) -> b.op.Outcome:

        failed = b_io.eh.badOutcome
        callParamsDict = {'myLinkedInBase': myLinkedInBase, 'vcardsDir': vcardsDir, }
        if self.invocationValidate(rtInv, cmndOutcome, callParamsDict, argsList).isProblematic():
            return failed(cmndOutcome)
        cmndArgsSpecDict = self.cmndArgsSpec()
        myLinkedInBase = csParam.mappedValue('myLinkedInBase', myLinkedInBase)
        vcardsDir = csParam.mappedValue('vcardsDir', vcardsDir)
####+END:
        self.cmndDocStr(f""" #+begin_org
** [[elisp:(org-cycle)][| *CmndDesc:* | ]]  Takes in Invitations.csv, augments vcards, returns count
        #+end_org """)

        inFile = self.cmndArgsGet("0", cmndArgsSpecDict, argsList)
        path_inFile = pathlib.Path(inFile).expanduser().resolve()
        if not path_inFile.exists(): return failed(cmndOutcome, f"Missing {path_inFile}")

        if vcardsDir is not None:
            path_vcardsDir = pathlib.Path(vcardsDir).expanduser().resolve(strict=True)

        logger.info(f"cmnd={vcardsInvitations} -- optPar:: vcardsDir={vcardsDir}  arg0:: Invitations.csv={path_inFile}")

        augmentation = invitations.LinkedInInvitations(path_inFile)
        augmentation.load()
        count = augmentation.augment_vcards(path_vcardsDir)

        return cmndOutcome.set(opResults=f"vcardsDir={path_vcardsDir} count={count}",)


####+BEGIN: b:py3:cs:method/args :methodName "cmndArgsSpec" :methodType "anyOrNone" :retType "bool" :deco "default" :argsList "self"
    """ #+begin_org
**  _[[elisp:(blee:menu-sel:outline:popupMenu)][±]]_ _[[elisp:(blee:menu-sel:navigation:popupMenu)][Ξ]]_ [[elisp:(outline-show-branches+toggle)][|=]] [[elisp:(bx:orgm:indirectBufOther)][|>]] *[[elisp:(blee:ppmm:org-mode-toggle)][|N]]*  Mtd-T-anyOrNone [[elisp:(outline-show-subtree+toggle)][||]] /cmndArgsSpec/ deco=default  deco=default  [[elisp:(org-cycle)][| ]]
    #+end_org """
    @cs.track(fnLoc=True, fnEntry=True, fnExit=True)
    def cmndArgsSpec(self, ):
####+END:
        """
***** Cmnd Args Specification
"""
        cmndArgsSpecDict = cs.CmndArgsSpecDict()

        cmndArgsSpecDict.argsDictAdd(
            argPosition="0",
            argName="inFile",
            argDefault='',
            argChoices=[],
            argDescription="Path to the Exported LinkedIn invitations file."
        )

        return cmndArgsSpecDict


####+BEGIN: b:py3:cs:cmnd/classHead :cmndName "vcardsMessages" :comment "" :extent "verify" :ro "cli" :parsMand "" :parsOpt "myLinkedInBase vcardsDir" :argsMin 1 :argsMax 1 :pyInv ""
""" #+begin_org
*  _[[elisp:(blee:menu-sel:outline:popupMenu)][±]]_ _[[elisp:(blee:menu-sel:navigation:popupMenu)][Ξ]]_ [[elisp:(outline-show-branches+toggle)][|=]] [[elisp:(bx:orgm:indirectBufOther)][|>]] *[[elisp:(blee:ppmm:org-mode-toggle)][|N]]*  CmndSvc-   [[elisp:(outline-show-subtree+toggle)][||]] <<vcardsMessages>>  =verify= parsOpt=myLinkedInBase vcardsDir argsMin=1 argsMax=1 ro=cli   [[elisp:(org-cycle)][| ]]
#+end_org """
class vcardsMessages(cs.Cmnd):
    cmndParamsMandatory = [ ]
    cmndParamsOptional = [ 'myLinkedInBase', 'vcardsDir', ]
    cmndArgsLen = {'Min': 1, 'Max': 1,}

    @cs.track(fnLoc=True, fnEntry=True, fnExit=True)
    def cmnd(self,
             rtInv: cs.RtInvoker,
             cmndOutcome: b.op.Outcome,
             myLinkedInBase: typing.Optional[str]=None,  # Cs Optional Param
             vcardsDir: typing.Optional[str]=None,  # Cs Optional Param
             argsList: typing.Optional[list[str]]=None,  # CsArgs
    ) -> b.op.Outcome:

        failed = b_io.eh.badOutcome
        callParamsDict = {'myLinkedInBase': myLinkedInBase, 'vcardsDir': vcardsDir, }
        if self.invocationValidate(rtInv, cmndOutcome, callParamsDict, argsList).isProblematic():
            return failed(cmndOutcome)
        cmndArgsSpecDict = self.cmndArgsSpec()
        myLinkedInBase = csParam.mappedValue('myLinkedInBase', myLinkedInBase)
        vcardsDir = csParam.mappedValue('vcardsDir', vcardsDir)
####+END:
        self.cmndDocStr(f""" #+begin_org
** [[elisp:(org-cycle)][| *CmndDesc:* | ]]  Takes in messages.csv, augments vcards, returns count
        #+end_org """)

        inFile = self.cmndArgsGet("0", cmndArgsSpecDict, argsList)
        path_inFile = pathlib.Path(inFile).expanduser().resolve()
        if not path_inFile.exists(): return failed(cmndOutcome, f"Missing {path_inFile}")

        if vcardsDir is not None:
            path_vcardsDir = pathlib.Path(vcardsDir).expanduser().resolve(strict=True)

        logger.info(f"cmnd={vcardsMessages} -- optPar:: vcardsDir={vcardsDir}  arg0:: Connections.csv={path_inFile}")

        augmentation = messages.LinkedInMessages(path_inFile)
        augmentation.load()
        augmentation.augment_vcards(path_vcardsDir)

        return cmndOutcome.set(opResults=path_vcardsDir,)



####+BEGIN: b:py3:cs:method/args :methodName "cmndArgsSpec" :methodType "anyOrNone" :retType "bool" :deco "default" :argsList "self"
    """ #+begin_org
**  _[[elisp:(blee:menu-sel:outline:popupMenu)][±]]_ _[[elisp:(blee:menu-sel:navigation:popupMenu)][Ξ]]_ [[elisp:(outline-show-branches+toggle)][|=]] [[elisp:(bx:orgm:indirectBufOther)][|>]] *[[elisp:(blee:ppmm:org-mode-toggle)][|N]]*  Mtd-T-anyOrNone [[elisp:(outline-show-subtree+toggle)][||]] /cmndArgsSpec/ deco=default  deco=default  [[elisp:(org-cycle)][| ]]
    #+end_org """
    @cs.track(fnLoc=True, fnEntry=True, fnExit=True)
    def cmndArgsSpec(self, ):
####+END:
        """
***** Cmnd Args Specification
"""
        cmndArgsSpecDict = cs.CmndArgsSpecDict()

        cmndArgsSpecDict.argsDictAdd(
            argPosition="0",
            argName="inFile",
            argDefault='',
            argChoices=[],
            argDescription="Path to the Exported LinkedIn messages.csv file."
        )

        return cmndArgsSpecDict


####+BEGIN: b:py3:cs:cmnd/classHead :cmndName "maildirMessages" :comment "" :extent "verify" :ro "cli" :parsMand "" :parsOpt "myLinkedInBase maildir" :argsMin 1 :argsMax 1 :pyInv ""
""" #+begin_org
*  _[[elisp:(blee:menu-sel:outline:popupMenu)][±]]_ _[[elisp:(blee:menu-sel:navigation:popupMenu)][Ξ]]_ [[elisp:(outline-show-branches+toggle)][|=]] [[elisp:(bx:orgm:indirectBufOther)][|>]] *[[elisp:(blee:ppmm:org-mode-toggle)][|N]]*  CmndSvc-   [[elisp:(outline-show-subtree+toggle)][||]] <<maildirMessages>>  =verify= parsOpt=myLinkedInBase maildir argsMin=1 argsMax=1 ro=cli   [[elisp:(org-cycle)][| ]]
#+end_org """
class maildirMessages(cs.Cmnd):
    cmndParamsMandatory = [ ]
    cmndParamsOptional = [ 'myLinkedInBase', 'maildir', ]
    cmndArgsLen = {'Min': 1, 'Max': 1,}

    @cs.track(fnLoc=True, fnEntry=True, fnExit=True)
    def cmnd(self,
             rtInv: cs.RtInvoker,
             cmndOutcome: b.op.Outcome,
             myLinkedInBase: typing.Optional[str]=None,  # Cs Optional Param
             maildir: typing.Optional[str]=None,  # Cs Optional Param
             argsList: typing.Optional[list[str]]=None,  # CsArgs
    ) -> b.op.Outcome:

        failed = b_io.eh.badOutcome
        callParamsDict = {'myLinkedInBase': myLinkedInBase, 'maildir': maildir, }
        if self.invocationValidate(rtInv, cmndOutcome, callParamsDict, argsList).isProblematic():
            return failed(cmndOutcome)
        cmndArgsSpecDict = self.cmndArgsSpec()
        myLinkedInBase = csParam.mappedValue('myLinkedInBase', myLinkedInBase)
        maildir = csParam.mappedValue('maildir', maildir)
####+END:
        self.cmndDocStr(f""" #+begin_org
** [[elisp:(org-cycle)][| *CmndDesc:* | ]]  Returns runFacterAndGetJsonOutputBytes.
        #+end_org """)


        inFile = self.cmndArgsGet("0", cmndArgsSpecDict, argsList)
        csv_path = pathlib.Path(inFile)
        if not csv_path.exists(): return failed(cmndOutcome, f"Missing {csv_path}")

        assert  csv_path.exists(), f"Missing {csv_path}"

        maildir_path = pathlib.Path(maildir)

        # telemetry = Telemetry("LinkedInMessages822")
        # converter = messages822.LinkedInMessagesToMaildir(csv_path, maildir_path, telemetry)
        converter = messagesMaildir.LinkedInMessagesToMaildir(csv_path, maildir_path)
        converter.run()

        return cmndOutcome.set(opResults=maildir_path,)


####+BEGIN: b:py3:cs:method/args :methodName "cmndArgsSpec" :methodType "anyOrNone" :retType "bool" :deco "default" :argsList "self"
    """ #+begin_org
**  _[[elisp:(blee:menu-sel:outline:popupMenu)][±]]_ _[[elisp:(blee:menu-sel:navigation:popupMenu)][Ξ]]_ [[elisp:(outline-show-branches+toggle)][|=]] [[elisp:(bx:orgm:indirectBufOther)][|>]] *[[elisp:(blee:ppmm:org-mode-toggle)][|N]]*  Mtd-T-anyOrNone [[elisp:(outline-show-subtree+toggle)][||]] /cmndArgsSpec/ deco=default  deco=default  [[elisp:(org-cycle)][| ]]
    #+end_org """
    @cs.track(fnLoc=True, fnEntry=True, fnExit=True)
    def cmndArgsSpec(self, ):
####+END:
        """
***** Cmnd Args Specification
"""
        cmndArgsSpecDict = cs.CmndArgsSpecDict()

        cmndArgsSpecDict.argsDictAdd(
            argPosition="0",
            argName="inFile",
            argDefault='',
            argChoices=[],
            argDescription="Path to the Exported LinkedIn connections file."
        )

        return cmndArgsSpecDict


####+BEGIN: b:py3:cs:framework/endOfFile :basedOn "classification"
""" #+begin_org
* [[elisp:(org-cycle)][| *End-Of-Editable-Text* |]] :: emacs and org variables and control parameters
#+end_org """

#+STARTUP: showall

### local variables:
### no-byte-compile: t
### end:
####+END:




from pathlib import Path as libpath
from typing import List, Dict
import csv
import vobject
import logging

# logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)
# logger.setLevel(logging.INFO)

class LinkedInInvitations:
    """
    Processes Invitations.csv to annotate vCards with invitation direction and message.
    """

    def __init__(self, csv_path: libpath):
        """
        Initialize with the path to Invitations.csv.
        """
        self.csv_path: libpath = libpath(csv_path).expanduser()
        self.invitations: List[Dict[str, str]] = []

    def load(self) -> None:
        """
        Load invitation records from CSV into memory.
        """
        with self.csv_path.open(newline='', encoding='utf-8') as csvfile:
            reader = csv.DictReader(csvfile)
            self.invitations = list(reader)
        logger.info(f"Loaded {len(self.invitations)} invitations.")

    def augment_vcards(self, vcard_dir: libpath) -> None:
        """
        Add invitation direction and message to matching vCards, avoiding duplication.
        """
        vcard_dir = libpath(vcard_dir).expanduser()
        if not self.invitations:
            self.load()

        for invitation in self.invitations:
            direction = invitation.get("Direction", "").strip().upper()
            message = invitation.get("Message", "").strip()
            profile_url = (
                invitation.get("inviteeProfileUrl") if direction == "OUTGOING"
                else invitation.get("inviterProfileUrl")
            )

            if not profile_url:
                logger.warning("Skipping invitation with missing profile URL.")
                continue

            uid = profile_url.rstrip('/').split('/')[-1]
            vcard_path = vcard_dir / f"{uid}.vcf"

            if not vcard_path.exists():
                logger.debug(f"vCard not found for {uid}, skipping.")
                continue

            with vcard_path.open('r', encoding='utf-8') as f:
                vcard = vobject.readOne(f.read())

            note_value = f"Invitation {direction.capitalize()}"
            if message:
                note_value += f" — {message}"

            # Check for existing identical note
            existing_note = vcard.note.value if 'note' in vcard.contents else ''
            if note_value in existing_note:
                logger.debug(f"Note already present for {uid}, skipping.")
                continue

            if 'note' in vcard.contents:
                vcard.note.value += f"\n{note_value}"
            else:
                vcard.add('note').value = note_value

            with vcard_path.open('w', encoding='utf-8') as f:
                f.write(vcard.serialize())
            logger.info(f"Updated vCard for {uid}.")




# -*- coding: utf-8 -*-

""" #+begin_org
* ~[Summary]~ :: A =CS-Unit= as equivalent of facter in py and remotely with rpyc.
#+end_org """

####+BEGIN: b:py3:cs:file/dblockControls :classification "cs-u"
""" #+begin_org
* [[elisp:(org-cycle)][| /Control Parameters Of This File/ |]] :: dblk ctrls classifications=cs-u
#+BEGIN_SRC emacs-lisp
(setq-local b:dblockControls t) ; (setq-local b:dblockControls nil)
(put 'b:dblockControls 'py3:cs:Classification "cs-u") ; one of cs-mu, cs-u, cs-lib, bpf-lib, pyLibPure
#+END_SRC
#+RESULTS:
: cs-u
#+end_org """
####+END:

####+BEGIN: b:prog:file/proclamations :outLevel 1
""" #+begin_org
* *[[elisp:(org-cycle)][| Proclamations |]]* :: Libre-Halaal Software --- Part Of BISOS ---  Poly-COMEEGA Format.
** This is Libre-Halaal Software. © Neda Communications, Inc. Subject to AGPL.
** It is part of BISOS (ByStar Internet Services OS)
** Best read and edited  with Blee in Poly-COMEEGA (Polymode Colaborative Org-Mode Enhance Emacs Generalized Authorship)
#+end_org """
####+END:

####+BEGIN: b:prog:file/particulars :authors ("./inserts/authors-mb.org")
""" #+begin_org
* *[[elisp:(org-cycle)][| Particulars |]]* :: Authors, version
** This File: /bisos/git/bxRepos/bisos-pip/myLinkedIn/py3/bisos/myLinkedIn/linkedinWeb_csu.py
** File True Name: /bisos/git/auth/bxRepos/bisos-pip/myLinkedIn/py3/bisos/myLinkedIn/linkedinWeb_csu.py
** Authors: Mohsen BANAN, http://mohsen.banan.1.byname.net/contact
#+end_org """
####+END:

####+BEGIN: b:py3:file/particulars-csInfo :status "inUse"
""" #+begin_org
* *[[elisp:(org-cycle)][| Particulars-csInfo |]]*
#+end_org """
import typing
csInfo: typing.Dict[str, typing.Any] = { 'moduleName': ['linkedinWeb_csu'], }
csInfo['version'] = '202505100515'
csInfo['status']  = 'inUse'
csInfo['panel'] = 'linkedinWeb_csu-Panel.org'
csInfo['groupingType'] = 'IcmGroupingType-pkged'
csInfo['cmndParts'] = 'IcmCmndParts[common] IcmCmndParts[param]'
####+END:

""" #+begin_org
* [[elisp:(org-cycle)][| ~Description~ |]] :: [[file:/bisos/git/auth/bxRepos/blee-binders/bisos-core/COMEEGA/_nodeBase_/fullUsagePanel-en.org][BISOS COMEEGA Panel]]
This a =Cs-Unit= for running the equivalent of facter in py and remotely with rpyc.
With BISOS, it is used in CMDB remotely.

** Relevant Panels:
** Status: In use with BISOS
** /[[elisp:(org-cycle)][| Planned Improvements |]]/ :
*** TODO complete fileName in particulars.
#+end_org """

####+BEGIN: b:prog:file/orgTopControls :outLevel 1
""" #+begin_org
* [[elisp:(org-cycle)][| Controls |]] :: [[elisp:(delete-other-windows)][(1)]] | [[elisp:(show-all)][Show-All]]  [[elisp:(org-shifttab)][Overview]]  [[elisp:(progn (org-shifttab) (org-content))][Content]] | [[file:Panel.org][Panel]] | [[elisp:(blee:ppmm:org-mode-toggle)][Nat]] | [[elisp:(bx:org:run-me)][Run]] | [[elisp:(bx:org:run-me-eml)][RunEml]] | [[elisp:(progn (save-buffer) (kill-buffer))][S&Q]]  [[elisp:(save-buffer)][Save]]  [[elisp:(kill-buffer)][Quit]] [[elisp:(org-cycle)][| ]]
** /Version Control/ ::  [[elisp:(call-interactively (quote cvs-update))][cvs-update]]  [[elisp:(vc-update)][vc-update]] | [[elisp:(bx:org:agenda:this-file-otherWin)][Agenda-List]]  [[elisp:(bx:org:todo:this-file-otherWin)][ToDo-List]]

#+end_org """
####+END:

####+BEGIN: b:py3:file/workbench :outLevel 1
""" #+begin_org
* [[elisp:(org-cycle)][| Workbench |]] :: [[elisp:(python-check (format "/bisos/venv/py3/bisos3/bin/python -m pyclbr %s" (bx:buf-fname))))][pyclbr]] || [[elisp:(python-check (format "/bisos/venv/py3/bisos3/bin/python -m pydoc ./%s" (bx:buf-fname))))][pydoc]] || [[elisp:(python-check (format "/bisos/pipx/bin/pyflakes %s" (bx:buf-fname)))][pyflakes]] | [[elisp:(python-check (format "/bisos/pipx/bin/pychecker %s" (bx:buf-fname))))][pychecker (executes)]] | [[elisp:(python-check (format "/bisos/pipx/bin/pycodestyle %s" (bx:buf-fname))))][pycodestyle]] | [[elisp:(python-check (format "/bisos/pipx/bin/flake8 %s" (bx:buf-fname))))][flake8]] | [[elisp:(python-check (format "/bisos/pipx/bin/pylint %s" (bx:buf-fname))))][pylint]]  [[elisp:(org-cycle)][| ]]
#+end_org """
####+END:

####+BEGIN: b:py3:cs:orgItem/basic :type "=PyImports= "  :title "*Py Library IMPORTS*" :comment "-- Framework and External Packages Imports"
""" #+begin_org
*  _[[elisp:(blee:menu-sel:outline:popupMenu)][±]]_ _[[elisp:(blee:menu-sel:navigation:popupMenu)][Ξ]]_ [[elisp:(outline-show-branches+toggle)][|=]] [[elisp:(bx:orgm:indirectBufOther)][|>]] *[[elisp:(blee:ppmm:org-mode-toggle)][|N]]*  =PyImports=  [[elisp:(outline-show-subtree+toggle)][||]] *Py Library IMPORTS* -- Framework and External Packages Imports  [[elisp:(org-cycle)][| ]]
#+end_org """
####+END:

# import os
import collections
# import invoke

####+BEGIN: b:py3:cs:framework/imports :basedOn "classification"
""" #+begin_org
*  _[[elisp:(blee:menu-sel:outline:popupMenu)][±]]_ _[[elisp:(blee:menu-sel:navigation:popupMenu)][Ξ]]_ [[elisp:(outline-show-branches+toggle)][|=]] [[elisp:(bx:orgm:indirectBufOther)][|>]] *[[elisp:(blee:ppmm:org-mode-toggle)][|N]]*  CsFrmWrk   [[elisp:(outline-show-subtree+toggle)][||]] *Imports* =Based on Classification=cs-u=
#+end_org """
from bisos import b
from bisos.b import cs
from bisos.b import b_io
from bisos.common import csParam

import collections
####+END:

# from bisos.facter import facter
# from bisos.banna import bannaPortNu

import pathlib
import re

import vobject
import random
import time

# from telemetry import Telemetry

from bisos.myLinkedIn import linkedinUtils
# from bisos.myLinkedIn import connections
from bisos.myLinkedIn import invitations
# from bisos.myLinkedIn import messages


# from bisos.myLinkedIn import messages_822

from bisos.myLinkedIn import linkedinWebInfo

import logging
logger = logging.getLogger(__name__)

####+BEGIN: b:py3:cs:orgItem/basic :type "=Executes=  "  :title "CSU-Lib Executions" :comment "-- cs.invOutcomeReportControl"
""" #+begin_org
*  _[[elisp:(blee:menu-sel:outline:popupMenu)][±]]_ _[[elisp:(blee:menu-sel:navigation:popupMenu)][Ξ]]_ [[elisp:(outline-show-branches+toggle)][|=]] [[elisp:(bx:orgm:indirectBufOther)][|>]] *[[elisp:(blee:ppmm:org-mode-toggle)][|N]]*  =Executes=   [[elisp:(outline-show-subtree+toggle)][||]] CSU-Lib Executions -- cs.invOutcomeReportControl  [[elisp:(org-cycle)][| ]]
#+end_org """
####+END:

cs.invOutcomeReportControl(cmnd=True, ro=True)

####+BEGIN: b:py3:cs:orgItem/section :title "Common Parameters Specification" :comment "based on cs.param.CmndParamDict -- As expected from CSU-s"
""" #+begin_org
*  _[[elisp:(blee:menu-sel:outline:popupMenu)][±]]_ _[[elisp:(blee:menu-sel:navigation:popupMenu)][Ξ]]_ [[elisp:(outline-show-branches+toggle)][|=]] [[elisp:(bx:orgm:indirectBufOther)][|>]] *[[elisp:(blee:ppmm:org-mode-toggle)][|N]]*  /Section/    [[elisp:(outline-show-subtree+toggle)][||]] *Common Parameters Specification* based on cs.param.CmndParamDict -- As expected from CSU-s  [[elisp:(org-cycle)][| ]]
#+end_org """
####+END:

def linkedinId_canonical(input: str,) -> str:
    return input


####+BEGIN: b:py3:cs:func/typing :funcName "commonParamsSpecify" :comment "~CSU Specification~" :funcType "ParSpc" :deco ""
""" #+begin_org
*  _[[elisp:(blee:menu-sel:outline:popupMenu)][±]]_ _[[elisp:(blee:menu-sel:navigation:popupMenu)][Ξ]]_ [[elisp:(outline-show-branches+toggle)][|=]] [[elisp:(bx:orgm:indirectBufOther)][|>]] *[[elisp:(blee:ppmm:org-mode-toggle)][|N]]*  F-T-ParSpc [[elisp:(outline-show-subtree+toggle)][||]] /commonParamsSpecify/  ~CSU Specification~  [[elisp:(org-cycle)][| ]]
#+end_org """
def commonParamsSpecify(
####+END:
        csParams: cs.param.CmndParamDict,
) -> None:
    # account, password and force come from commonCsParams
    csParams.parDictAdd(
        parName='browser',
        parDescription="Browser to be used.",
        parDataType=None,
        parDefault=None,
        parChoices=[],
        argparseShortOpt=None,
        argparseLongOpt='--browser',
    )
    csParams.parDictAdd(
        parName='browserProfilePath',
        parDescription="Profile name within Chrome.",
        parDataType=None,
        parDefault="~/.config/google-chrome/Default",
        parChoices=[],
        argparseShortOpt=None,
        argparseLongOpt='--browserProfilePath',
    )
    csParams.parDictAdd(
        parName='minInterval',
        parDescription="Minimum Interval in Seconds.",
        parDataType=None,
        parDefault=10,
        parChoices=[],
        argparseShortOpt=None,
        argparseLongOpt='--minInterval',
    )
    csParams.parDictAdd(
        parName='maxInterval',
        parDescription="Maximum Interval in Seconds.",
        parDataType=None,
        parDefault=22,
        parChoices=[],
        argparseShortOpt=None,
        argparseLongOpt='--maxInterval',
    )


####+BEGIN: blee:bxPanel:foldingSection :outLevel 0 :sep nil :title "Direct Command Services" :anchor ""  :extraInfo "Examples and CSs"
""" #+begin_org
*  _[[elisp:(blee:menu-sel:outline:popupMenu)][±]]_ _[[elisp:(blee:menu-sel:navigation:popupMenu)][Ξ]]_ [[elisp:(outline-show-branches+toggle)][|=]] [[elisp:(bx:orgm:indirectBufOther)][|>]] *[[elisp:(blee:ppmm:org-mode-toggle)][|N]]*     [[elisp:(outline-show-subtree+toggle)][| _Direct Command Services_: |]]  Examples and CSs  [[elisp:(org-shifttab)][<)]] E|
#+end_org """
####+END:

####+BEGIN: b:py3:cs:cmnd/classHead :cmndName "examples_csu" :comment "" :parsMand "" :parsOpt "" :argsMin 0 :argsMax 0 :pyInv ""
""" #+begin_org
*  _[[elisp:(blee:menu-sel:outline:popupMenu)][±]]_ _[[elisp:(blee:menu-sel:navigation:popupMenu)][Ξ]]_ [[elisp:(outline-show-branches+toggle)][|=]] [[elisp:(bx:orgm:indirectBufOther)][|>]] *[[elisp:(blee:ppmm:org-mode-toggle)][|N]]*  CmndSvc-   [[elisp:(outline-show-subtree+toggle)][||]] <<examples_csu>>  =verify= ro=cli   [[elisp:(org-cycle)][| ]]
#+end_org """
class examples_csu(cs.Cmnd):
    cmndParamsMandatory = [ ]
    cmndParamsOptional = [ ]
    cmndArgsLen = {'Min': 0, 'Max': 0,}

    @cs.track(fnLoc=True, fnEntry=True, fnExit=True)
    def cmnd(self,
             rtInv: cs.RtInvoker,
             cmndOutcome: b.op.Outcome,
    ) -> b.op.Outcome:

        failed = b_io.eh.badOutcome
        callParamsDict = {}
        if self.invocationValidate(rtInv, cmndOutcome, callParamsDict, None).isProblematic():
            return failed(cmndOutcome)
####+END:
        self.cmndDocStr(f""" #+begin_org
** [[elisp:(org-cycle)][| *CmndDesc:* | ]]  Basic example command.
        #+end_org """)

        myName = cs.G.icmMyName()

        od = collections.OrderedDict
        cmnd = cs.examples.cmndEnter
        literal = cs.examples.execInsert

        credsPars = od([('account', "someUser"), ('password', "somePasswd")])

        oneLinkedinUrl = '''https://www.linkedin.com/in/first-last-1234'''
        oneLinkedinId =  linkedinUtils.LinkedinId.fromUrl(oneLinkedinUrl)
        oneVCardsDir = "~/bpos/usageEnvs/selected/myLinkedIn/selected/VCards"

        cs.examples.menuChapter('=LinkedIn Web Info --- Exit All Chrome Windows=')

        cs.examples.menuSection('/Show Selected as target of symlink/')

        literal(f"readlink -f {oneVCardsDir}")
        literal(f"ls -l {oneVCardsDir}")

        cs.examples.menuSection('/vcardNeedsUpdate:: Filters stdin to stdout -- All output Needs Update/')

        literal(f"ls {oneVCardsDir}/* | {myName} -i vcardNeedsUpdate | wc ")

        cmnd('vcardNeedsUpdate',
             wrapper=f"ls {oneVCardsDir}/* | ",
            )

        cmnd('vcardNeedsUpdate',
             pars=od([('verbosity', '20'),]),
             wrapper=f"ls {oneVCardsDir}/* | head -10 | ",
            )

        cs.examples.menuSection('/contactInfoToVCard::  From Qualifier, absPath, basename, id or URL/')

        cs.examples.menuSubSection('/contactInfoToVCard::  From LinkedinId -- Close All Chrome Frames/')

        cmnd('contactInfoToVCard',
             pars=od([('vcardsDir', oneVCardsDir),]),
             args=oneLinkedinId)

        cmnd('contactInfoToVCard',
             pars=od([('vcardsDir', oneVCardsDir), ('account', "someUser"), ('password', "somePasswd")]),
             wrapper=f"ls -1 {oneVCardsDir} | egrep '.vcf$' | head -2 | sed 's/\.[^.]*$//' | ",
            )

        cs.examples.menuSubSection('/contactInfoToVCard::  From absPath -- Close All Chrome Frames/')

        cmnd('contactInfoToVCard',
             pars=od([('vcardsDir', oneVCardsDir), ('account', "someUser"), ('password', "somePasswd")]),
             wrapper=f"ls {oneVCardsDir} | head -2 ",
            )

        cs.examples.menuSubSection('/contactInfoToVCard::  From absPath with vcardNeedsUpdate -- Close All Chrome Frames/')

        cmnd('contactInfoToVCard',
             pars=od([('account', "someUser"), ('password', "somePasswd")]),
             wrapper=f"ls {oneVCardsDir}/* | {myName} -i vcardNeedsUpdate | head -4 | ",
            )

        cmnd('contactInfoToVCard',
             pars=od([('verbosity', '20'), ('account', "someUser"), ('password', "somePasswd")]),
             wrapper=f"ls {oneVCardsDir}/* | {myName} -i vcardNeedsUpdate | head -2 | ",
            )

        return(cmndOutcome)


####+BEGIN: b:py3:cs:cmnd/classHead :cmndName "vcardNeedsUpdate" :comment "" :extent "verify" :ro "cli" :parsMand "" :parsOpt "vcardsDir" :argsMin 0 :argsMax 9999 :pyInv ""
""" #+begin_org
*  _[[elisp:(blee:menu-sel:outline:popupMenu)][±]]_ _[[elisp:(blee:menu-sel:navigation:popupMenu)][Ξ]]_ [[elisp:(outline-show-branches+toggle)][|=]] [[elisp:(bx:orgm:indirectBufOther)][|>]] *[[elisp:(blee:ppmm:org-mode-toggle)][|N]]*  CmndSvc-   [[elisp:(outline-show-subtree+toggle)][||]] <<vcardNeedsUpdate>>  =verify= parsOpt=vcardsDir argsMax=9999 ro=cli   [[elisp:(org-cycle)][| ]]
#+end_org """
class vcardNeedsUpdate(cs.Cmnd):
    cmndParamsMandatory = [ ]
    cmndParamsOptional = [ 'vcardsDir', ]
    cmndArgsLen = {'Min': 0, 'Max': 9999,}

    @cs.track(fnLoc=True, fnEntry=True, fnExit=True)
    def cmnd(self,
             rtInv: cs.RtInvoker,
             cmndOutcome: b.op.Outcome,
             vcardsDir: typing.Optional[str]=None,  # Cs Optional Param
             argsList: typing.Optional[list[str]]=None,  # CsArgs
    ) -> b.op.Outcome:

        failed = b_io.eh.badOutcome
        callParamsDict = {'vcardsDir': vcardsDir, }
        if self.invocationValidate(rtInv, cmndOutcome, callParamsDict, argsList).isProblematic():
            return failed(cmndOutcome)
        cmndArgsSpecDict = self.cmndArgsSpec()
        vcardsDir = csParam.mappedValue('vcardsDir', vcardsDir)
####+END:
        self.cmndDocStr(f""" #+begin_org
** [[elisp:(org-cycle)][| *CmndDesc:* | ]]  Takes input from stdin or args, for each vcard updates email if needed.
        #+end_org """)

        cmndArgs = self.cmndArgsGet("0&9999", cmndArgsSpecDict, argsList)

        if vcardsDir is not None:
            path_vcardsDir = pathlib.Path(vcardsDir).expanduser().resolve(strict=False)

        def processArgsAndStdin(cmndArgs, process):

            for each in cmndArgs:
                process(each)
            stdinArgs = b_io.stdin.readAsList()
            for each in stdinArgs:
                process(each)

        def process(linkedinQualifier):

            vcardPath = linkedinUtils.LinkedinQualifier.toVCardPath(linkedinQualifier, path_vcardsDir)
            if vcardPath is None:
                logger.error(f"Bad Input -- linkedinQualifier={linkedinQualifier}")
                return

            if linkedinUtils.VCard.needs_update(vcardPath) == False:
                logger.info(f"Does not need updating -- skipped: {vcardPath}")
            else:
                print(vcardPath)

        processArgsAndStdin(cmndArgs, process)

        return cmndOutcome.set(opResults=None,)


####+BEGIN: b:py3:cs:method/args :methodName "cmndArgsSpec" :methodType "anyOrNone" :retType "bool" :deco "default" :argsList "self"
    """ #+begin_org
**  _[[elisp:(blee:menu-sel:outline:popupMenu)][±]]_ _[[elisp:(blee:menu-sel:navigation:popupMenu)][Ξ]]_ [[elisp:(outline-show-branches+toggle)][|=]] [[elisp:(bx:orgm:indirectBufOther)][|>]] *[[elisp:(blee:ppmm:org-mode-toggle)][|N]]*  Mtd-T-anyOrNone [[elisp:(outline-show-subtree+toggle)][||]] /cmndArgsSpec/ deco=default  deco=default  [[elisp:(org-cycle)][| ]]
    #+end_org """
    @cs.track(fnLoc=True, fnEntry=True, fnExit=True)
    def cmndArgsSpec(self, ):
####+END:
        """
***** Cmnd Args Specification
"""
        cmndArgsSpecDict = cs.CmndArgsSpecDict()

        cmndArgsSpecDict.argsDictAdd(
            argPosition="0&9999",
            argName="cmndArgs",
            argDefault=None,
            argChoices=[],
            argDescription="Path to the LinkedIn Basic_LinkedInDataExport_DATE.zip file."
        )

        return cmndArgsSpecDict



####+BEGIN: b:py3:cs:cmnd/classHead :cmndName "contactInfoToVCard" :comment "" :extent "verify" :ro "cli" :parsMand "" :parsOpt "vcardsDir account password browser minInterval maxInterval force" :argsMin 0 :argsMax 9999 :pyInv ""
""" #+begin_org
*  _[[elisp:(blee:menu-sel:outline:popupMenu)][±]]_ _[[elisp:(blee:menu-sel:navigation:popupMenu)][Ξ]]_ [[elisp:(outline-show-branches+toggle)][|=]] [[elisp:(bx:orgm:indirectBufOther)][|>]] *[[elisp:(blee:ppmm:org-mode-toggle)][|N]]*  CmndSvc-   [[elisp:(outline-show-subtree+toggle)][||]] <<contactInfoToVCard>>  =verify= parsOpt=vcardsDir account password browser minInterval maxInterval force argsMax=9999 ro=cli   [[elisp:(org-cycle)][| ]]
#+end_org """
class contactInfoToVCard(cs.Cmnd):
    cmndParamsMandatory = [ ]
    cmndParamsOptional = [ 'vcardsDir', 'account', 'password', 'browser', 'minInterval', 'maxInterval', 'force', ]
    cmndArgsLen = {'Min': 0, 'Max': 9999,}

    @cs.track(fnLoc=True, fnEntry=True, fnExit=True)
    def cmnd(self,
             rtInv: cs.RtInvoker,
             cmndOutcome: b.op.Outcome,
             vcardsDir: typing.Optional[str]=None,  # Cs Optional Param
             account: typing.Optional[str]=None,  # Cs Optional Param
             password: typing.Optional[str]=None,  # Cs Optional Param
             browser: typing.Optional[str]=None,  # Cs Optional Param
             minInterval: typing.Optional[str]=None,  # Cs Optional Param
             maxInterval: typing.Optional[str]=None,  # Cs Optional Param
             force: typing.Optional[str]=None,  # Cs Optional Param
             argsList: typing.Optional[list[str]]=None,  # CsArgs
    ) -> b.op.Outcome:

        failed = b_io.eh.badOutcome
        callParamsDict = {'vcardsDir': vcardsDir, 'account': account, 'password': password, 'browser': browser, 'minInterval': minInterval, 'maxInterval': maxInterval, 'force': force, }
        if self.invocationValidate(rtInv, cmndOutcome, callParamsDict, argsList).isProblematic():
            return failed(cmndOutcome)
        cmndArgsSpecDict = self.cmndArgsSpec()
        vcardsDir = csParam.mappedValue('vcardsDir', vcardsDir)
        account = csParam.mappedValue('account', account)
        password = csParam.mappedValue('password', password)
        browser = csParam.mappedValue('browser', browser)
        minInterval = csParam.mappedValue('minInterval', minInterval)
        maxInterval = csParam.mappedValue('maxInterval', maxInterval)
        force = csParam.mappedValue('force', force)
####+END:
        self.cmndDocStr(f""" #+begin_org
** [[elisp:(org-cycle)][| *CmndDesc:* | ]]  Takes input from stdin or args, for each vcard updates email if needed.
        #+end_org """)

        cmndArgs = self.cmndArgsGet("0&9999", cmndArgsSpecDict, argsList)

        if force is not False:
            force = True

        if vcardsDir is not None:
            path_vcardsDir = pathlib.Path(vcardsDir).expanduser().resolve(strict=False)

        cmndArgsTotal = len(cmndArgs)
        stdinArgsTotal = 0
        cmndArgsProcessed = 0
        stdinArgsProcessed = 0
        webAccessCount = 0
        alreadyProcessedCount = 0

        def processArgsAndStdin(cmndArgs, process):
            nonlocal cmndArgsProcessed, stdinArgsProcessed, stdinArgsTotal

            for each in cmndArgs:
                cmndArgsProcessed += 1
                process(each)
            stdinArgs = b_io.stdin.readAsList()
            stdinArgsTotal = len(stdinArgs)
            # if the list was sorted, we shuffle it
            random.shuffle(stdinArgs)
            for each in stdinArgs:
                stdinArgsProcessed += 1
                process(each)

        augmentor = linkedinWebInfo.LinkedInRemoteAugmentor(
            chrome_user_data_dir=pathlib.Path("~/.config/google-chrome").expanduser(),
            chrome_profile="Default"
        )

        augmentor.start_driver(account, password)

        def process(linkedinQualifier):
            nonlocal webAccessCount, alreadyProcessedCount

            vcardPath = linkedinUtils.LinkedinQualifier.toVCardPath(linkedinQualifier, path_vcardsDir)
            if vcardPath is None:
                logger.error(f"Bad Input -- linkedinQualifier={linkedinQualifier}")
                return

            linkedinId = linkedinUtils.LinkedinQualifier.asLinkedInId(vcardPath)
            waitBeforeNext = augmentVcardFromLinkedin(augmentor, linkedinId, vcardsDir, force=force)
            if waitBeforeNext == False:
                alreadyProcessedCount += 1
                return

            webAccessCount += 1
            waitInterval = random.randint(minInterval, maxInterval)
            logger.info(f"Waiting for waitInterval={waitInterval} Seconds -- webRetrieved={webAccessCount} alreadyProcessed={alreadyProcessedCount} -- stdinArgsProcessed={stdinArgsProcessed} of {stdinArgsTotal} -- cmndArgsProcessed={cmndArgsProcessed} of {cmndArgsTotal}")
            if (cmndArgsProcessed + stdinArgsProcessed) == (stdinArgsTotal + cmndArgsTotal):
                logger.info(f"Final Count Reached::  stdinArgsProcessed={stdinArgsProcessed} of {stdinArgsTotal} -- cmndArgsProcessed={cmndArgsProcessed} of {cmndArgsTotal}")
                return
            time.sleep(waitInterval)

        processArgsAndStdin(cmndArgs, process)

        augmentor.stop_driver()

        resultsSummary = f"webRetrieved={webAccessCount} alreadyProcessed={alreadyProcessedCount} -- stdinArgs={stdinArgsProcessed} of {stdinArgsTotal} -- cmndArgs={cmndArgsProcessed} of {cmndArgsTotal}"

        return cmndOutcome.set(opResults=resultsSummary,)


####+BEGIN: b:py3:cs:method/args :methodName "cmndArgsSpec" :methodType "anyOrNone" :retType "bool" :deco "default" :argsList "self"
    """ #+begin_org
**  _[[elisp:(blee:menu-sel:outline:popupMenu)][±]]_ _[[elisp:(blee:menu-sel:navigation:popupMenu)][Ξ]]_ [[elisp:(outline-show-branches+toggle)][|=]] [[elisp:(bx:orgm:indirectBufOther)][|>]] *[[elisp:(blee:ppmm:org-mode-toggle)][|N]]*  Mtd-T-anyOrNone [[elisp:(outline-show-subtree+toggle)][||]] /cmndArgsSpec/ deco=default  deco=default  [[elisp:(org-cycle)][| ]]
    #+end_org """
    @cs.track(fnLoc=True, fnEntry=True, fnExit=True)
    def cmndArgsSpec(self, ):
####+END:
        """
***** Cmnd Args Specification
"""
        cmndArgsSpecDict = cs.CmndArgsSpecDict()

        cmndArgsSpecDict.argsDictAdd(
            argPosition="0&9999",
            argName="cmndArgs",
            argDefault=None,
            argChoices=[],
            argDescription="Path to the LinkedIn Basic_LinkedInDataExport_DATE.zip file."
        )

        return cmndArgsSpecDict


####+BEGIN: blee:bxPanel:foldingSection :outLevel 0 :sep nil :title "Support Functions" :anchor ""  :extraInfo ""
""" #+begin_org
*  _[[elisp:(blee:menu-sel:outline:popupMenu)][±]]_ _[[elisp:(blee:menu-sel:navigation:popupMenu)][Ξ]]_ [[elisp:(outline-show-branches+toggle)][|=]] [[elisp:(bx:orgm:indirectBufOther)][|>]] *[[elisp:(blee:ppmm:org-mode-toggle)][|N]]*     [[elisp:(outline-show-subtree+toggle)][| _Support Functions_: |]]    [[elisp:(org-shifttab)][<)]] E|
#+end_org """
####+END:

####+BEGIN: b:py3:cs:func/typing :funcName "augmentVcardFromLinkedin" :funcType "extTyped" :deco "track"
""" #+begin_org
*  _[[elisp:(blee:menu-sel:outline:popupMenu)][±]]_ _[[elisp:(blee:menu-sel:navigation:popupMenu)][Ξ]]_ [[elisp:(outline-show-branches+toggle)][|=]] [[elisp:(bx:orgm:indirectBufOther)][|>]] *[[elisp:(blee:ppmm:org-mode-toggle)][|N]]*  F-T-extTyped [[elisp:(outline-show-subtree+toggle)][||]] /augmentVcardFromLinkedin/  deco=track  [[elisp:(org-cycle)][| ]]
#+end_org """
@cs.track(fnLoc=True, fnEntry=True, fnExit=True)
def augmentVcardFromLinkedin(
####+END:
        augmentor: linkedinWebInfo.LinkedInRemoteAugmentor,
        linkedinId: str,
        vcardsDir: pathlib.Path,
        force: bool=False
    ) -> bool:
    """ #+begin_org
** [[elisp:(org-cycle)][| *DocStr | ] Extracts contact info from LinkedIn and augments the corresponding vCard file.
The vCard file is assumed to be named <linkedinId>.vcf inside vcard_dir.
#+end_org """

    waitBeforeNext = True

    linkedinUrl = linkedinUtils.LinkedinId.toUrl(linkedinId)
    vcardsdDirPath = pathlib.Path(f"{vcardsDir}").expanduser().resolve()

    vcardPath = linkedinUtils.VCard.find_vcard(vcardsdDirPath, linkedinId)

    if vcardPath ==  None:
        logger.info(f"Missing {linkedinId} VCard Skipped")
        waitBeforeNext = False
        return waitBeforeNext

    if force == False:
        if linkedinUtils.VCard.needs_update(vcardPath) == False:
            logger.info(f"Does not need updating -- skipped: {vcardPath}")
            waitBeforeNext = False
            return waitBeforeNext

    logger.info(f"Augmenting vCard from LinkedIn URL: {linkedinUrl}")
    contact_info = augmentor.fetch_contact_info(linkedinUrl)

    linkedinUtils.VCard.augment_vcard_with_contact_info(vcardPath, contact_info)
    waitBeforeNext = True
    return waitBeforeNext

####+BEGIN: b:py3:cs:framework/endOfFile :basedOn "classification"
""" #+begin_org
* [[elisp:(org-cycle)][| *End-Of-Editable-Text* |]] :: emacs and org variables and control parameters
#+end_org """

#+STARTUP: showall

### local variables:
### no-byte-compile: t
### end:
####+END:



import time
import logging
from pathlib import Path
from typing import Optional, Dict
from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.chrome.service import Service as ChromeService
from webdriver_manager.chrome import ChromeDriverManager
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.common.exceptions import TimeoutException, NoSuchElementException
import vobject

import urllib.parse

logger = logging.getLogger(__name__)
# logger.setLevel(logging.INFO)

class LinkedInRemoteAugmentor:
    """
    Automates LinkedIn profile contact info extraction using an existing Chrome login session.

    Requirements:
    - User must be already logged into LinkedIn in Chrome.
    - Chrome must be closed before starting this script (profile cannot be in use).
    """

    def __init__(self, chrome_user_data_dir: Path, chrome_profile: str = "Default"):
        """
        Initialize with path to Chrome user data and profile.

        :param chrome_user_data_dir: Base directory for Chrome user data (e.g., ~/.config/google-chrome)
        :param chrome_profile: Profile name within Chrome (e.g., 'Default', 'Profile 1')
        """
        self.chrome_user_data_dir = chrome_user_data_dir
        self.chrome_profile = chrome_profile
        self.driver: Optional[webdriver.Chrome] = None

    def start_driver(self, username: Optional[str] = None, password: Optional[str] = None) -> None:
        """
        Start Chrome WebDriver and ensure LinkedIn login.
        Reuses existing session if already logged in; otherwise, logs in using credentials.
        """
        logger.info("Launching Chrome with profile reuse...")

        options = webdriver.ChromeOptions()
        options.add_argument(f"--user-data-dir={self.chrome_user_data_dir}")
        options.add_argument(f"--profile-directory={self.chrome_profile}")
        options.add_argument("--start-maximized")

        self.driver = webdriver.Chrome(
            service=ChromeService(ChromeDriverManager().install()),
            options=options
        )

        logger.info("Chrome WebDriver started.")
        self.driver.get("https://www.linkedin.com/feed/")
        time.sleep(3)

        # Check if we are already logged in
        if "login" in self.driver.current_url:
            logger.info("Not logged in — attempting login.")
            if not username or not password:
                raise ValueError("Login required but no credentials provided.")
            self.driver.get("https://www.linkedin.com/login")

            try:
                email_field = self.driver.find_element(By.ID, "username")
                password_field = self.driver.find_element(By.ID, "password")
                email_field.send_keys(username)
                password_field.send_keys(password)
                password_field.send_keys(Keys.RETURN)
                time.sleep(5)
                logger.info("Login attempt complete.")
            except NoSuchElementException:
                logger.error("Login page elements not found. Login failed.")
        else:
            logger.info("Already logged in — continuing session.")


    def fetch_contact_info(self, linkedin_url: str) -> Dict[str, Optional[str]]:
        """
        Extract contact info (email, phone, website, etc.) from a LinkedIn profile's contact-info overlay.
        """
        if not self.driver:
            raise RuntimeError("WebDriver is not started. Call start_driver() first.")

        logger.info(f"Visiting LinkedIn profile: {linkedin_url}")
        self.driver.get(linkedin_url)
        time.sleep(3)

        contact_info: Dict[str, Optional[str]] = {
            "email": None,
            "phone": None,
            "website": None,
            "twitter": None,
            "address": None,
            "birthday": None,
            "profile_url": None,
        }

        try:
            contact_info_button = WebDriverWait(self.driver, 5).until(
                EC.element_to_be_clickable((By.ID, "top-card-text-details-contact-info"))
            )
            contact_info_button.click()
            logger.debug("Clicked on contact info button.")

            #     DEBUGGING ---  Kept for Evolution
            # logger.debug("Capture the HTML after clicking")
            # html = self.driver.page_source
            # with open("/tmp/linkedin_modal_debug.html", "w") as f:
            #      f.write(html)
            
            time.sleep(2)
        except Exception as e:
            logger.warning(f"Could not click contact info button: {e}")
            return contact_info

        # Parse visible contact fields
        try:
            modal = WebDriverWait(self.driver, 5).until(
                EC.presence_of_element_located((By.CLASS_NAME, "artdeco-modal__content"))
            )

            links = modal.find_elements(By.TAG_NAME, "a")
            for link in links:
                href = link.get_attribute("href") or ""
                text = link.text.strip()

                #
                # NOTYET
                # June 14, 2025 Only email extraction works for now.
                #
                
                if href.startswith("mailto:"):
                    contact_info["email"] = text
                elif href.startswith("tel:"):
                    contact_info["phone"] = text
                elif "twitter.com" in href:
                    contact_info["twitter"] = href
                elif "linkedin.com/in/" in href:
                    contact_info["profile_url"] = href
                elif href.startswith("http"):
                    contact_info["website"] = href

            # Extract text spans for address and birthday
            spans = modal.find_elements(By.TAG_NAME, "span")
            for span in spans:
                text = span.text.strip()
                if "Birthday" in text:
                    contact_info["birthday"] = text.replace("Birthday", "").strip()
                elif "Address" in text:
                    contact_info["address"] = text.replace("Address", "").strip()

            logger.debug(f"Extracted contact info: {contact_info}")

        except Exception as e:
            logger.warning(f"Error extracting contact info details: {e}")

        return contact_info

    def stop_driver(self) -> None:
        """
        Stop the Chrome WebDriver and close browser.
        """
        if self.driver:
            logger.info("Closing Chrome WebDriver.")
            self.driver.quit()
            self.driver = None




from pathlib import Path
from typing import List, Dict, Optional
import csv
import vobject


import logging
logger = logging.getLogger(__name__)


class LinkedInConnections:
    """
    Parses LinkedIn Connections CSV and generates vCards.
    """

    def __init__(self, csv_path: Path):
        """
        Initialize with path to LinkedIn Connections.csv.
        """
        self.csv_path: Path = csv_path
        self.connections: List[Dict[str, str]] = []

    def load(self) -> None:
        """
        Load connection records from CSV, skipping header comments.
        """
        with self.csv_path.open(newline='', encoding='utf-8') as f:
            for _ in range(3):
                next(f)
            reader = csv.DictReader(f)
            self.connections = list(reader)

    def create_vcards(self, output_dir: Path) -> int:
        """
        Generate vCard files for each connection.
        """
        if not self.connections:
            self.load()

        output_dir.mkdir(parents=True, exist_ok=True)

        counter = 0

        for entry in self.connections:

            counter += 1

            vcard = vobject.vCard()
            vcard.add('fn').value = f"{entry['First Name']} {entry['Last Name']}"
            vcard.add('n').value = vobject.vcard.Name(
                family=entry['Last Name'], given=entry['First Name']
            )
            vcard.add('note').value = f"Connected On: {entry['Connected On']}"
            vcard.add('org').value = [entry.get('Company', '')]
            vcard.add('title').value = entry.get('Position', '')

            url: Optional[str] = entry.get('URL', '')
            uid = url.rstrip('/').split('/')[-1] if url else f"{entry['First Name']}_{entry['Last Name']}"
            filename: Path = output_dir / f"{uid}.vcf"

            logger.debug(filename)
            with filename.open('w', encoding='utf-8') as f:
                vcf_str = vcard.serialize()
                logger.debug(vcf_str)
                f.write(vcf_str)

        return counter



import csv
import logging
from pathlib import Path
from collections import defaultdict
from typing import List, Dict, DefaultDict

import vobject

logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)
# logging.basicConfig(level=logging.INFO)

class LinkedInMessages:
    """
    Processes Messages.csv to annotate vCards with message history from first-level connections.
    """

    def __init__(self, csv_path: Path):
        """
        Initialize with the path to Messages.csv.
        """
        self.csv_path: Path = csv_path
        self.messages_by_uid: DefaultDict[str, List[str]] = defaultdict(list)

    def load(self) -> None:
        """
        Load and group messages by LinkedIn user ID extracted from SENDER or RECIPIENT PROFILE URL.
        """
        with self.csv_path.open(newline='', encoding='utf-8') as csvfile:
            reader = csv.DictReader(csvfile)
            for row in reader:
                sender_url = row.get("SENDER PROFILE URL", "").strip()
                recipient_url = row.get("RECIPIENT PROFILE URLS", "").strip()
                profile_url = sender_url or recipient_url
                if not profile_url:
                    logger.warning("Skipping message with no profile URL: %s", row)
                    continue

                uid = profile_url.rstrip('/').split('/')[-1]
                sender = row.get("FROM", "").strip()
                date = row.get("DATE", "").strip()
                content = row.get("CONTENT", "").strip()

                message = f"{date} — {sender}: {content}"
                self.messages_by_uid[uid].append(message)

    def augment_vcards(self, vcard_dir: Path) -> None:
        """
        Add messages to vCards using custom X-Message fields.
        """
        if not self.messages_by_uid:
            self.load()

        for uid, messages in self.messages_by_uid.items():
            vcard_path = vcard_dir / f"{uid}.vcf"
            if not vcard_path.exists():
                logger.info("Skipping missing vCard: %s", vcard_path)
                continue

            with vcard_path.open('r', encoding='utf-8') as f:
                vcard = vobject.readOne(f.read())

            existing_messages = {
                line.value for line in vcard.contents.get('x-message', [])
            }

            new_messages = [
                msg for msg in messages if msg not in existing_messages
            ]

            if not new_messages:
                logger.debug("No new messages for %s", uid)
                continue

            for msg in new_messages:
                xmsg = vcard.add('x-message')
                xmsg.value = msg

            with vcard_path.open('w', encoding='utf-8') as f:
                f.write(vcard.serialize())

            logger.info("Updated vCard with %d new messages: %s", len(new_messages), vcard_path)



import csv
import logging
from pathlib import Path
import zipfile
import vobject
from typing import Optional, List, Dict
from urllib.parse import urlparse

from datetime import datetime

logger = logging.getLogger(__name__)
# logger.setLevel(logging.INFO)

class LinkedinId:
    """
    Utility functions for handling LinkedinId
    """

    @staticmethod
    def fromUrl(url: str) -> str:
        """
        Extract the LinkedIn ID from the profile URL.
        """
        parts = url.rstrip('/').split('/')
        return parts[-1]

    @staticmethod
    def toPath( uid: str, vcard_dir: Path,) -> Optional[Path]:
        """
        Find the vCard file corresponding to the LinkedIn ID (UID) in the directory.
        """

        assert vcard_dir
        vcard_path = vcard_dir / f"{uid}.vcf"
        if vcard_path.exists():
            return vcard_path
        else:
            # vcard_path = vcard_dir / f"EXTRA_{uid}.vcf"
            return None

    @staticmethod
    def toUrl(id: str) -> str:
        """
        Convert linkedinId to linkedin Url.
        """
        prefix = "EXTRA_"
        if id.startswith(prefix):
            id =  id[len(prefix):]

        return f"https://www.linkedin.com/in/{id}"


class LinkedinQualifier:
    """
    A LinkedInQualifer is one of:

        - LinkedInVCardAbsPath   : starts with / ends with /LinkedInVCardBasename
        - LinkedInVCardBasename  : non-directory part of file path formed as id.vcf
        - LinkedInId             : The id part of https://www.linkedin.com/in/{id}
        - LinkedInUrl            : The URL in the form of

   LinkedInQualifer is then combined with:
        - vcardsDir              : dirname in which  LinkedInVCardBasename resides
                                 : vcardsDir = LinkedInVCardAbsPath -  LinkedInVCardBasename

    LinkedInVCardPath is a pathlib.Path object that is used for conversions and processing

    Usage is like this:
           LinkedInVCardPath = LinkedinQualifier.toVCardPath(qualifier, vcardsDir)
    And then
           linkedInId = LinkedinQualifier.asLinkedInId(vcardPath)
    """

    @staticmethod
    def toVCardPath(qualifier: str, vcardsDir: str) -> Optional[Path]:
        """
        """
        assert qualifier
        assert vcardsDir

        vcardsDirPath = Path(vcardsDir)
        # if not vcardsDirPath.exists():
        # raise b.exception.badUsage()

        vcardPath = None

        # LinkedInUrl -- Check if the input is a URL
        try:
            result = urlparse(qualifier)
            if all([result.scheme, result.netloc]):
                id = LinkedinId.fromUrl(qualifier)
                vcardPath = LinkedinId.toPath(id, vcardsDirPath)
                if vcardPath is not None:
                    return vcardPath
                vcardPath = VCard.create_extra_vcard(vcardsDirPath, id)
                if vcardPath is not None:
                    return vcardPath
                else:
                    logger.error(f"VCard.create_extra_vcard failed for {id}")
                    return None

        except Exception:
            pass

        # Check if the input is a file path
        qualifierPath = Path(qualifier)

        # LinkedInVCardAbsPath
        if qualifierPath.is_absolute():
            if qualifierPath.exists():
                return qualifierPath
            else:
                logger.info(f"Missing {qualifierPath}")
                id =  LinkedinQualifier.asLinkedInId(qualifierPath)
                vcardPath = VCard.create_extra_vcard(vcardsDirPath, id)
                if vcardPath is not None:
                    return vcardPath
                else:
                    logger.error(f"VCard.create_extra_vcard failed for {id}")
                    return None

        # LinkedInVCardBasename
        if qualifierPath.suffix == '.vcf':
            vcardPath = vcardsDir  / qualifierPath
            if vcardPath.exists():
                return vcardPath
            else:
                logger.info(f"Missing {vcardPath}")
                id =  LinkedinQualifier.asLinkedInId(qualifierPath)
                vcardPath = VCard.create_extra_vcard(vcardsDirPath, id)
                if vcardPath is not None:
                    return vcardPath
                else:
                    logger.error(f"VCard.create_extra_vcard failed for {id}")
                    return None

        # LinkedInId
        vcardPath = LinkedinId.toPath(qualifier, vcardsDirPath)
        if vcardPath is not None:
            return vcardPath
        vcardPath = VCard.create_extra_vcard(vcardsDirPath, qualifier)
        if vcardPath is not None:
            return vcardPath
        else:
            logger.error(f"VCard.create_extra_vcard failed for {qualifier}")
            return None


    @staticmethod
    def asLinkedInId(vcardPath: Path) -> str:
        """
        """
        stem = vcardPath.stem
        return stem


class VCard:
    """
    Utility functions for handling vCards and LinkedIn data files.
    """

    @staticmethod
    def read_csv(file_path: Path) -> List[Dict[str, str]]:
        """
        Read a CSV file and return the rows as a list of dictionaries.
        """
        if not file_path.exists():
            raise FileNotFoundError(f"{file_path} does not exist.")

        with file_path.open('r', encoding='utf-8') as f:
            return list(csv.DictReader(f))

    @staticmethod
    def write_vcard(vcard, vcard_path: Path) -> None:
        """
        Write the vCard object to a file.
        """
        with vcard_path.open('w', encoding='utf-8') as vcard_file:
            vcard_file.write(vcard.serialize())

    @staticmethod
    def read_vcard(vcard_path: Path):
        """
        Read a vCard from a file and return the vCard object.
        """
        with vcard_path.open('r', encoding='utf-8') as vcard_file:
            return vobject.readOne(vcard_file.read())

    @staticmethod
    def create_extra_vcard(vcard_dir: Path, uid: str) -> Optional[Path]:
        """
        Create a new vCard file corresponding to the LinkedIn ID (UID) in the directory.
        Assume that the LinkedinId is in the form of first-last-uniqId
        Create a blank vcard using vobject.
        Add to it:
            - firstname (from uid)
            - lastname (from uid)
            - linkinedUrl (based on uid)
        Write the created vcard using vobject.
        Return Path to created vcard.
        """
        vcard_path = vcard_dir / f"EXTRA_{uid}.vcf"
        if vcard_path.exists():
            return vcard_path
        # Split the UID to extract first and last names
        parts = uid.split('-')
        if len(parts) < 2:
            logger.error(f"Invalid UID format: {uid}")
            return None

        first_name, last_name = parts[0], parts[1]

        # Create a new vCard
        vcard = vobject.vCard()
        vcard.add('fn').value = f"{first_name} {last_name}"
        vcard.add('n').value = vobject.vcard.Name(family=last_name, given=first_name)
        vcard.add('url').value = LinkedinId.toUrl(uid)

        # Write the vCard to a file
        VCard.write_vcard(vcard, vcard_path)
        logger.info(f"Created extra vCard at: {vcard_path}")

        return vcard_path


    @staticmethod
    def find_vcard(vcard_dir: Path, uid: str) -> Optional[Path]:
        """
        Find the vCard file corresponding to the LinkedIn ID (UID) in the directory.
        """
        vcard_path = vcard_dir / f"{uid}.vcf"
        if vcard_path.exists():
            return vcard_path
        return None

    @staticmethod
    def needs_update(vcardPath: Path) -> bool:
        """
        Find the vCard file corresponding to the LinkedIn ID (UID) in the directory.
        """

        vcardStr  = vcardPath.read_text(encoding="utf-8").strip()
        vcard = vobject.readOne(vcardStr)

        field = "x-date"
        if hasattr(vcard, field):
            value = vcard.contents[field][0].value
            logger.info(f"vcardPath={vcardPath} -- {field} = {value} -- Skipped")
            return False
        
        field = "email"
        if hasattr(vcard, field):
            value = vcard.contents[field][0].value
            logger.info(f"vcardPath={vcardPath} -- {field} = {value} -- Skipped")
            return False

        return True

    @staticmethod
    def update_or_add_custom_field(card, field_name, value):
        # Check if the field exists
        existing_field = next((comp for comp in card.contents.get(field_name, [])), None)

        if existing_field:
            # If the field exists, replace its value
            existing_field.value = value
        else:
            # If the field does not exist, add it
            custom_field = card.add(field_name)
            custom_field.value = value


    @staticmethod
    def augment_vcard_with_contact_info(vcard_path: Path, contact_info: Dict[str, Optional[str]]) -> None:
        """
        Augments an existing vCard file with extracted LinkedIn contact info.
        """
        logger.info(f"Augmenting vCard at: {vcard_path}")

        if not vcard_path.exists():
            logger.error(f"vCard not found: {vcard_path}")
            return

        vcard_str = vcard_path.read_text(encoding="utf-8").strip()
        vcard = vobject.readOne(vcard_str)

        field_mapping = {
            "email": ("email", "INTERNET"),
            "phone": ("tel", "CELL"),
            "website": ("url", None),
            "twitter": ("x-twitter", None),
            "address": ("adr", None),
            "birthday": ("bday", None),
            "profile_url": ("x-linkedin", None),
        }

        for key, (field, type_param) in field_mapping.items():
            value = contact_info.get(key)
            if value:
                if hasattr(vcard, field):
                    vcard.contents[field][0].value = value
                else:
                    new_field = vcard.add(field)
                    new_field.value = value
                    if type_param:
                        new_field.type_param = type_param
                logger.debug(f"Updated {field} with: {value}")

        VCard.update_or_add_custom_field(vcard, 'x-date', datetime.now().strftime("%Y%m%d%H%M%S"))
        # vcard.add('x-date').value = datetime.now().strftime("%Y%m%d%H%M%S")
                
        vcard_path.write_text(vcard.serialize(), encoding="utf-8")
        logger.info("vCard updated.")


class Common:

    @staticmethod
    def unzip_file(zip_path: Path, extract_to: Path) -> None:
        """Unzips a .zip file to the specified directory using pathlib.
        Use it like so: unzip_file(Path("LinkedInDataExport.zip"), Path("unzipped"))
        """
        logger.info(f"Unzipping {zip_path} to {extract_to}")
        extract_to.mkdir(parents=True, exist_ok=True)
        with zipfile.ZipFile(zip_path, 'r') as zip_ref:
            zip_ref.extractall(extract_to)



#!/usr/bin/env python

# Template File:  /bisos/apps/defaults/begin/templates/purposed/pyModule/python/setup.py
# Blee Panel: /bisos/git/auth/bxRepos/blee-binders/bisos-core/bisos-pip/bisos.pycs/pipPackaging/_nodeBase_/fullUsagePanel-en.org

# b:py3:pypi:setup/pkgName Arguments  :pkgName "somePkg" :pkgNameSpace "bisos"
####+BEGIN: b:py3:pypi:setup/pkgName :comment "Auto Detected"

import setuptools
import re
import inspect
import pathlib

def pkgName():
    ''' From this eg., filepath=.../bisos-pip/PkgName/py3/setup.py, extract PkgName. '''
    filename = inspect.getframeinfo(inspect.currentframe()).filename
    grandMother = pathlib.Path(filename).resolve().parent.parent.name
    return f'bisos.{grandMother}'

def description():
    ''' Extract title from ./_description.org which is expected to have a #+title: line. '''
    try:
        with open('./_description.org') as file:
            while line := file.readline():
                if match := re.search(r'^#\+title: (.*)',  line.rstrip()):
                    return match.group(1)
            return 'MISSING TITLE in ./_description.org'
    except IOError:
        return  'ERROR: Could not read ./_description.org file.'

def longDescription():
    ''' Read README.rst as a string. '''
    fileName = './README.rst'
    try:
        with open(fileName) as file:
           result = file.read()
        return result
    except IOError:
        return  f'ERROR: Could not read {fileName} file.'

####+END:

# b:py3:pypi:setup/version Arguments  :forSys t :forPyPi t :constant "666"
####+BEGIN: b:py3:pypi:setup/version :comment "Auto Detected"

# ./pypiUploadVer EXISTED -- forPypiVersion=0.13 -- forLocalVersion=0.1 -- constant=NA
def pkgVersion():
        return '0.13'

####+END:

# b:py3:pypi:setup/requires :extras ; :requirements "requirements.txt" (bring here requirements.txt)
####+BEGIN: b:py3:pypi:setup/requires :extras ("selenium" "vobject" "requests" "beautifulsoup4" "webdriver_manager")

requires = [
"bisos",
"bisos.b",
"bisos.common",
"bisos.csPlayer",
"selenium",
"vobject",
"requests",
"beautifulsoup4",
"webdriver_manager",
"setuptools==75.8.0",
]
####+END:

# b:py3:pypi:setup/scripts :comment
####+BEGIN: b:py3:pypi:setup/scripts :comment ""

scripts = [
'bin/myLinkedIn.cs',
'bin/myLinkedInWeb.cs',
]
####+END:

# b:py3:pypi:setup/dataFiles :comment
####+BEGIN: b:py3:pypi:setup/dataFiles :comment "Instead of ./MANIFEST.in or in pyproject.toml"

data_files = [
(' ',  ['lh-agpl3-LICENSE.txt', '_description.org', 'README.rst']),
]
####+END:

# :pkgName "--auto--"  --- results in use of name=pkgName(),
####+BEGIN: b:py3:pypi:setup/funcArgs :comment "defaults to --auto--"

setuptools.setup(
    name='bisos.myLinkedIn',  # 'bisos.myLinkedIn'
    version=pkgVersion(),
    packages=setuptools.find_packages(),
    scripts=scripts,
    data_files=data_files,
    include_package_data=True,
    zip_safe=False,
    author='Mohsen Banan',
    author_email='libre@mohsen.1.banan.byname.net',
    maintainer='Mohsen Banan',
    maintainer_email='libre@mohsen.1.banan.byname.net',
    license='AGPL',
    description=description(),
    long_description=longDescription(),
    install_requires=requires,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Affero General Public License v3',
        'Operating System :: POSIX',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        ]
    )

####+END:



import csv
import mailbox
import logging
from email.message import EmailMessage
from pathlib import Path
from typing import Optional
# from telemetry import Telemetry

class LinkedInMessagesToMaildir:
    """
    Converts LinkedIn Messages.csv into Maildir-format RFC822 messages.
    """

    def __init__(self, csv_path: Path, maildir_path: Path,
                 #telemetry: Optional[Telemetry] = None):
                 ):
        self.csv_path = csv_path.expanduser().resolve()
        self.maildir_path = maildir_path.expanduser().resolve()
        #self.telemetry = telemetry or Telemetry("LinkedInMessagesToMaildir")
        self.logger = logging.getLogger(__name__)

    def load_messages(self) -> list[dict]:
        self.logger.debug(f"Loading messages from: {self.csv_path}")
        # with self.telemetry.span("load_csv"), self.csv_path.open(encoding="utf-8") as f:
        with self.csv_path.open(encoding="utf-8") as f:
            reader = csv.DictReader(f)
            return list(reader)

    def create_email(self, row: dict) -> EmailMessage:
        msg = EmailMessage()
        msg["From"] = f"{row['FROM']} <{row.get('SENDER PROFILE URL', '').strip() or 'unknown@linkedin.com'}>"
        msg["To"] = f"{row['TO']} <{row.get('RECIPIENT PROFILE URLS', '').strip() or 'unknown@linkedin.com'}>"
        msg["Date"] = row.get("DATE", "")
        msg["Subject"] = row.get("SUBJECT", "No Subject").strip() or "No Subject"

        body = row.get("CONTENT", "").strip()
        msg.set_content(body)
        return msg

    def export_to_maildir(self):
        self.logger.info(f"Exporting messages to maildir: {self.maildir_path}")

        # with self.telemetry.span("export_maildir"):
        #
        maildir = mailbox.Maildir(self.maildir_path, create=True)

        messages = self.load_messages()
        count = 0
        for row in messages:
            try:
                msg = self.create_email(row)
                maildir.add(msg)
                count += 1
            except Exception as e:
                self.logger.warning(f"Failed to convert row to mail: {e}")

        self.logger.info(f"Exported {count} messages.")

    def run(self):
        self.export_to_maildir()
